@@ -3,7 +3,7 @@
 %%
 %%     Arithmetic on vectors and points (represented as three-tuples).
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -260,27 +260,11 @@ average({V10,V11,V12}, {V20,V21,V22}, {V30,V31,V32}, {V40,V41,V42})
     L = 0.25,
     {L*(V10+V20+V30+V40),L*(V11+V21+V31+V41),L*(V12+V22+V32+V42)}.
 
--spec bounding_box([e3d_vector()]) -> [e3d_vector()].
-    
-bounding_box([{X,Y,Z}|Vs]) ->
-    bounding_box_1(Vs, X, X, Y, Y, Z, Z).
 
-bounding_box_1([{X,_,_}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when X < X0 ->
-    bounding_box_1(Vs, X, X1, Y0, Y1, Z0, Z1);
-bounding_box_1([{X,_,_}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when X > X1 ->
-    bounding_box_1(Vs, X0, X, Y0, Y1, Z0, Z1);
-bounding_box_1([{_,Y,_}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when Y < Y0 ->
-    bounding_box_1(Vs, X0, X1, Y, Y1, Z0, Z1);
-bounding_box_1([{_,Y,_}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when Y > Y1 ->
-    bounding_box_1(Vs, X0, X1, Y0, Y, Z0, Z1);
-bounding_box_1([{_,_,Z}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when Z < Z0 ->
-    bounding_box_1(Vs, X0, X1, Y0, Y1, Z, Z1);
-bounding_box_1([{_,_,Z}|_]=Vs, X0, X1, Y0, Y1, Z0, Z1) when Z > Z1 ->
-    bounding_box_1(Vs, X0, X1, Y0, Y1, Z0, Z);
-bounding_box_1([_|Vs], X0, X1, Y0, Y1, Z0, Z1) ->
-    bounding_box_1(Vs, X0, X1, Y0, Y1, Z0, Z1);
-bounding_box_1([], X0, X1, Y0, Y1, Z0, Z1) ->
-    [{X0,Y0,Z0},{X1,Y1,Z1}].
+%% Should be removed and calls should be changed to e3d_bv instead.
+-spec bounding_box([e3d_vector()]) -> [e3d_vector()].
+bounding_box(List) when is_list(List) ->
+    tuple_to_list(e3d_bv:box(List)).
 
 -spec degrees(e3d_vector(), e3d_vector()) -> float().
     
