@@ -3,7 +3,7 @@
 %%
 %%     Plug-in for absolute commands -> move and snap
 %%
-%%  Copyright (c) 2006-2009 Andrzej Giniewicz
+%%  Copyright (c) 2006-2011 Andrzej Giniewicz
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -174,7 +174,7 @@ check_mirror([{Obj,VSet}|Rest],Shs) ->
     end.
 
 mirror_error() ->
-    wings_u:error(?__(1,"You cannot move vertices from mirror plane")).
+    wings_u:error_msg(?__(1,"You cannot move vertices from mirror plane")).
 
 get_selection(#st{selmode=SelMode}=St) ->
     #st{sel=Sel} = case SelMode of
