@@ -3,7 +3,7 @@
 %%
 %%     Image plane plug-in
 %%
-%%  Copyright (c) 2002-2008 Bjorn Gustavsson.
+%%  Copyright (c) 2002-2011 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -25,14 +25,14 @@ init() ->
     true.
 
 menu({shape}, Menu) ->
-    insert_before_more(Menu);
+    insert_before_image(Menu);
 menu(_, Menu) -> Menu.
 
-insert_before_more([H|_]=More) when element(1, element(2, H)) == more ->
-    [image_menu(),separator|More];
-insert_before_more([H|T]) ->
-    [H|insert_before_more(T)];
-insert_before_more([]) ->
+insert_before_image([{_,image,_}=Image|Rest]) ->
+    [Image,image_menu(),separator|Rest];
+insert_before_image([H|T]) ->
+    [H|insert_before_image(T)];
+insert_before_image([]) ->
     [image_menu()].
 
 image_menu() ->
@@ -54,40 +54,50 @@ make_image(Name) ->
 	#e3d_image{}=Image ->
 	    make_image_1(Name, Image);
 	{error,Error} ->
-	    wpa:error(?__(1,"Failed to load \"~s\": ~s\n"),
+	    wpa:error_msg(?__(1,"Failed to load \"~s\": ~s\n"),
 		      [Name,file:format_error(Error)])
     end.
 
 make_image_1(Name0, #e3d_image{type=Type}=Image0) ->
     %% Convert to the format that wings_image wants before padding (faster).
-    Image1 = e3d_image:convert(Image0, img_type(Type), 1, lower_left),
-    Name = filename:rootname(filename:basename(Name0)),
-    #e3d_image{width=W0,height=H0} = Image1,
-    Image = case pad_image(Image1) of
-		Image1 ->
-		    %% The image was not padded, therefore it is safe
-		    %% to keep the image as external.
-		    Image0;
-		Image2 ->
-		    %% The image has been padded. We must make sure that
-		    %% the new image is saved - so force it to be internal.
-		    Image2#e3d_image{filename=none}
-	    end,
-    #e3d_image{width=W,height=H} = Image,
-    ImageId = wings_image:new(Name, Image),
-    case can_texture_be_loaded(Image) of
-	false ->
-	    wpa:error(?__(1,"The image cannot be loaded as a texture (it is probably too large)."));
-	true ->
+	case wings_image:maybe_exceds_opengl_caps(Image0) of
+    {error,GlErr} ->
+	    wpa:error_msg(?__(1,"The image cannot be loaded as a texture.~nFile: \"~s\"~n GLU Error: ~p - ~s~n"),
+		      [Name0,GlErr, glu:errorString(GlErr)]);
+	Image0i -> 
+		Image1 = e3d_image:convert(Image0i, img_type(Type), 1, lower_left),
+		Name = filename:rootname(filename:basename(Name0)),
+		#e3d_image{width=W0,height=H0} = Image1,
+		Image = case pad_image(Image1) of
+			Image1 ->
+				%% The image was not padded, therefore it is safe
+				%% to keep the image as external.
+				Image0;
+			Image2 ->
+				%% The image has been padded. We must make sure that
+				%% the new image is saved - so force it to be internal.
+				Image2#e3d_image{filename=none}
+			end,
+		#e3d_image{width=W,height=H} = Image,
+		ImageId = wings_image:new(Name, Image),
 	    MaxU = W0/W,
 	    MaxV = H0/H,
 	    M = [image],
-	    Fs = [#e3d_face{vs=[0,3,2,1],tx=[1,0,3,2],mat=M},
-		  #e3d_face{vs=[1,2,3,0],tx=[2,3,0,1],mat=M}],
-	    UVs = [{0.0,0.0},{MaxU,0.0},{MaxU,MaxV},{0.0,MaxV}],
+	    Fs = [#e3d_face{vs=[0,3,2,1]},
+		  #e3d_face{vs=[2,3,7,6],tx=[6,2,3,7],mat=M},
+		  #e3d_face{vs=[0,4,7,3]},
+		  #e3d_face{vs=[1,2,6,5]},
+		  #e3d_face{vs=[4,5,6,7]},
+		  #e3d_face{vs=[0,1,5,4],tx=[4,0,1,5],mat=M}],
+	    UVs = [{0.0,MaxV},{MaxU,MaxV},{0.0,0.0},{MaxU,0.0},
+	           {0.0,0.0},{MaxU,0.0},{0.0,MaxV},{MaxU,MaxV}],
+	    HardEdges = [{0,3},{2,3},{1,2},{0,1},{3,7},{6,7},
+	                 {2,6},{0,4},{4,7},{4,5},{5,6},{1,5}],
 	    {X,Y} = ratio(W0, H0),
-	    Vs = [{0.0,-Y,-X},{0.0,Y,-X},{0.0,Y,X},{0.0,-Y,X}],
-	    Mesh = #e3d_mesh{type=polygon,fs=Fs,vs=Vs,tx=UVs},
+	    D = 1.0e-3/2,
+	    Vs = [{-D,-Y,X},{-D,Y,X},{D,Y,X},{D,-Y,X},
+	          {-D,-Y,-X},{-D,Y,-X},{D,Y,-X},{D,-Y,-X}],
+	    Mesh = #e3d_mesh{type=polygon,fs=Fs,vs=Vs,tx=UVs,he=HardEdges},
 	    Obj = #e3d_object{obj=Mesh},
 	    White = wings_color:white(),
 	    Black = wings_color:white(),
@@ -108,12 +118,6 @@ ratio(D, D) -> {1.0,1.0};
 ratio(W, H) when W < H -> {1.0,H/W};
 ratio(W, H) -> {W/H,1.0}.
     
-can_texture_be_loaded(#e3d_image{width=W,height=H,image=Pixels}) ->
-    gl:texImage2D(?GL_PROXY_TEXTURE_2D, 0, ?GL_RGB,
-		  W, H, 0, ?GL_RGB, ?GL_UNSIGNED_BYTE, Pixels),
-    W == gl:getTexLevelParameteriv(?GL_PROXY_TEXTURE_2D, 0,
-				   ?GL_TEXTURE_WIDTH).
-
 pad_image(#e3d_image{width=W0,image=Pixels0,bytes_pp=PP}=Image) ->
     case nearest_power_two(W0) of
 	W0 ->
