@@ -3,7 +3,7 @@
 %%
 %%     This module draws objects using OpenGL.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -15,7 +15,7 @@
 -export([refresh_dlists/1,
 	 invalidate_dlists/1,
 	 update_sel_dlist/0,
-	 changed_we/2,
+	 changed_we/2, update_normals/1,
 	 split/3,original_we/1,update_dynamic/2,join/1,abort_split/1]).
 
 % export for plugins (and wings_proxy) that need to draw stuff
