@@ -3,7 +3,7 @@
 %%
 %%     This module contains most edge command and edge utility functions.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson.
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -20,7 +20,8 @@
 	 cut/3,fast_cut/3,screaming_cut/3,
 	 dissolve_edges/2,dissolve_edge/2,
 	 hardness/3,
-	 patch_edge/4,patch_edge/5]).
+	 patch_edge/4,patch_edge/5,
+	 select_nth_ring/2]).
 
 -export([dissolve_isolated_vs/2]).
 
@@ -162,19 +163,37 @@ screaming_cut(Edge, NewVPos, We0) ->
 dissolve_edge(Edge, We) ->
     dissolve_edges([Edge], We).
 
-dissolve_edges(Edges0, We0) when is_list(Edges0) ->
-    #we{es=Etab} = We1 = foldl(fun internal_dissolve_edge/2, We0, Edges0),
-    case [E || E <- Edges0, array:get(E, Etab) =/= undefined] of
-	Edges0 ->
-	    %% No edge was deleted in the last pass. We are done.
-	    We = wings_we:rebuild(We0),
-	    wings_we:validate_mirror(We);
-	Edges ->
-	    dissolve_edges(Edges, We1)
-    end;
+dissolve_edges(Edges, We) when is_list(Edges) ->
+    Faces = gb_sets:to_list(wings_face:from_edges(Edges, We)),
+    dissolve_edges(Edges, Faces, We);
 dissolve_edges(Edges, We) ->
     dissolve_edges(gb_sets:to_list(Edges), We).
 
+dissolve_edges(Edges0, Faces, We0) when is_list(Edges0) ->
+    #we{es=Etab} = We1 = foldl(fun internal_dissolve_edge/2, We0, Edges0),
+    case [E || E <- Edges0, array:get(E, Etab) =/= undefined] of
+        Edges0 ->
+            %% No edge was deleted in the last pass. We are done.
+            We2 = wings_we:rebuild(We0),
+            #we{fs=Ftab}=We = wings_we:validate_mirror(We2),
+            lists:foreach(fun(Face) ->
+                case gb_trees:is_defined(Face, Ftab) of
+                    true ->
+                        case wings_we:is_face_consistent(Face, We) of
+                            true ->
+                                ok;
+                            false ->
+                                wings_u:error_msg(?__(1,"Dissolving would cause a badly formed face."))
+                        end;
+                    false ->
+                        ok
+                end
+            end, Faces),
+            We;
+        Edges ->
+            dissolve_edges(Edges, Faces, We1)
+    end.
+
 internal_dissolve_edge(Edge, #we{es=Etab}=We0) ->
     case array:get(Edge, Etab) of
 	undefined -> We0;
@@ -294,13 +313,7 @@ dissolve_edge_2(Edge, FaceRemove, FaceKeep,
 	    internal_dissolve_edge(AnEdge, We);
 	#edge{rf=FaceKeep,rtpr=Same,rtsu=Same} ->
 	    internal_dissolve_edge(AnEdge, We);
-	_Other ->
-	    case wings_we:is_face_consistent(FaceKeep, We) of
-		true ->
-		    We;
-		false ->
-		    wings_u:error(?__(1,"Dissolving would cause a badly formed face."))
-	    end
+	_Other -> We
     end.
 
 %% dissolve_isolated_vs([Vertex], We) -> We'
@@ -852,3 +865,109 @@ patch_edge(Edge, ToEdge, Face, OrigEdge, Etab) ->
 		  R#edge{rtpr=ToEdge}
 	  end,
     array:set(Edge, New, Etab).
+
+%%%% Select every nth ring
+select_nth_ring(N, #st{selmode=edge}=St) ->
+    Sel = wings_sel:fold(fun(Edges, #we{id=Id}=We, ObjAcc) ->
+                EdgeRings = nth_ring_1(Edges, {N,N}, We, Edges, gb_sets:new()),
+                Sel0 = wings_we:visible_edges(EdgeRings, We),
+                [{Id,Sel0}|ObjAcc]
+        end,[],St),
+    wings_sel:set(Sel, St);
+select_nth_ring(_N, St) ->
+    St.
+
+nth_ring_1(Edges0, N, We, OrigEs, Acc) ->
+    case gb_sets:is_empty(Edges0) of
+      true -> Acc;
+      false ->
+        {Edge,Edges1} = gb_sets:take_smallest(Edges0),
+        Rings = nth_ring_2(Edge, N, We, OrigEs, gb_sets:singleton(Edge)),
+        Edges = gb_sets:subtract(Edges1, Rings),
+        nth_ring_1(Edges, N, We, OrigEs, gb_sets:union(Rings,Acc))
+    end.
+
+nth_ring_2(Edge, {N,Int}, We, OrigEs, Acc) ->
+    case opposing_edge(Edge, We, left) of
+      unknown ->
+        case opposing_edge(Edge, We, right) of
+          unknown ->
+            Acc;
+          NextEdge ->
+            {_,Edges0} = nth_ring_3(NextEdge,Edge,Edge,{N-1,Int},right,left,We,OrigEs,Acc),
+            Edges0
+        end;
+      NextEdge ->
+        {Check0,Edges0} = case gb_sets:is_member(NextEdge,OrigEs) of
+          true -> {stop,Acc};
+          false -> nth_ring_3(NextEdge,Edge,Edge,{N-1,Int},left,right,We,OrigEs,Acc)
+        end,
+        case opposing_edge(Edge, We, right) of
+          unknown ->
+            Edges0;
+          PrevEdge ->
+            {Check1,Edges1} = case gb_sets:is_member(PrevEdge,OrigEs) of
+              true -> {stop,Acc};
+              false -> nth_ring_3(PrevEdge,Edge,Edge,{N-1,Int},right,left,We,OrigEs,Acc)
+            end,
+            nth_ring_4(Check0,Edges0,Check1,Edges1,OrigEs)
+        end
+    end.
+
+nth_ring_3(CurEdge,PrevEdge,LastEdge,{0,N},Side,Oposite,We,OrigEs,Acc0) ->
+    Acc = gb_sets:insert(CurEdge,Acc0),
+    nth_ring_3(CurEdge,PrevEdge,LastEdge,{N,N},Side,Oposite,We,OrigEs,Acc);
+nth_ring_3(CurEdge,PrevEdge,LastEdge,{N0,Int},Side,Oposite,We,OrigEs,Acc) ->
+    case opposing_edge(CurEdge, We, Side) of
+      unknown -> {ok,Acc};
+      PrevEdge ->
+        case opposing_edge(CurEdge, We, Oposite) of
+          unknown -> {ok,Acc};
+          PrevEdge -> {ok,Acc};
+          LastEdge -> {ok,Acc};
+          NextEdge ->
+            case gb_sets:is_member(NextEdge,OrigEs) of
+              true ->
+                {stop,Acc};
+              false ->
+                nth_ring_3(NextEdge,CurEdge,LastEdge,{N0-1,Int},Oposite,Side,We,OrigEs,Acc)
+            end
+        end;
+      LastEdge -> {ok,Acc};
+      NextEdge ->
+        case gb_sets:is_member(NextEdge,OrigEs) of
+          true ->
+            {stop,Acc};
+          false ->
+            nth_ring_3(NextEdge,CurEdge,LastEdge,{N0-1,Int},Side,Oposite,We,OrigEs,Acc)
+        end
+    end.
+
+nth_ring_4(_,Edges,_,Edges,_) ->
+    Edges;
+nth_ring_4(Check0,Edges0,Check1,Edges1,OrigEs) ->
+    S0 = gb_sets:size(Edges0),
+    S1 = gb_sets:size(Edges1),
+    if
+      S0 =:= 1 -> Edges1;
+      S1 =:= 1 -> Edges0;
+      true -> nth_ring_5(Check0,S0,Edges0,Check1,S1,Edges1,OrigEs)
+    end.
+
+nth_ring_5(ok,_,Edges0,ok,_,Edges1,_) ->
+    gb_sets:union(Edges0,Edges1);
+nth_ring_5(stop,S0,Edges0,stop,S1,Edges1,_) ->
+    case S0 > S1 of
+      true -> Edges1;
+      false -> Edges0
+    end;
+nth_ring_5(stop,_,Edges0,_,_,Edges1,OrigEs) ->
+    case gb_sets:is_subset(Edges0,OrigEs) of
+      true -> Edges1;
+      false -> Edges0
+    end;
+nth_ring_5(_,_,Edges0,stop,_,Edges1,OrigEs) ->
+    case gb_sets:is_subset(Edges1,OrigEs) of
+      true -> Edges0;
+      false -> Edges1
+    end.
