@@ -3,7 +3,7 @@
 %%
 %%     Import and export of Nendo .ndo files.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -192,7 +192,7 @@ export(Name, #st{shapes=Shapes0}=St) ->
 check_size(#we{name=Name,es=Etab}) ->
     case wings_util:array_entries(Etab) of
 	Sz when Sz > 65535 ->
-	    wings_u:error(?__(1,"Object \"")
+	    wings_u:error_msg(?__(1,"Object \"")
 			  ++Name
 			  ++?__(2,"\" cannot be exported ")
 			  ++?__(3,"to Nendo format (too many edges)."));
