@@ -3,7 +3,7 @@
 %%
 %%     This module contains the commands in the File menu.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -41,7 +41,14 @@ export_filename(Prop0, #st{file=File}, Cont) ->
 
 %% import_filename([Prop], Continuation).
 %%   The Continuation fun will be called like this: Continuation(Filename).
-import_filename(Ps0, Cont) ->
+import_filename(Prop, Cont) ->
+    case get(wings_not_running) of
+	undefined ->
+	    import_filename_1(Prop, Cont);
+	{import, FileName} ->
+	    Cont(FileName)
+    end.
+import_filename_1(Ps0, Cont) ->
     This = wings_wm:this(),
     Dir = wings_pref:get_value(current_directory),
     String = case os:type() of
@@ -65,7 +72,15 @@ import_filename(Ps0, Cont) ->
 
 %% export_filename([Prop], Continuation).
 %%   The Continuation fun will be called like this: Continuation(Filename).
-export_filename(Prop0, Cont) ->
+export_filename(Prop, Cont) ->
+    case get(wings_not_running) of
+	undefined -> 
+	    export_filename_1(Prop, Cont);
+	{export, FileName} ->
+	    Cont(FileName)
+    end.
+			 
+export_filename_1(Prop0, Cont) ->
     This = wings_wm:this(),
     Dir = wings_pref:get_value(current_directory),
     Prop = Prop0 ++ [{directory,Dir}],
@@ -83,10 +98,10 @@ export_filename(Prop0, Cont) ->
 		  end
 	  end,
     String = case os:type() of
-        {win32,_} -> "Export";
-        _Other    -> ?__(1,"Export")
-	end,
-	wings_plugin:call_ui({file,save_dialog,Prop++[{title,String}],Fun}).
+		 {win32,_} -> "Export";
+		 _Other    -> ?__(1,"Export")
+	     end,
+    wings_plugin:call_ui({file,save_dialog,Prop++[{title,String}],Fun}).
 
 init() ->
     wings_pref:set_default(save_unused_materials,false),
@@ -124,10 +139,10 @@ menu(_) ->
       ?__(14,"Save only the selected objects or faces")},
      {?__(15,"Save Incrementally"),save_incr,
       ?__(26,"Generate new filename and save")},
-	  %% if there are more options we'll make a panel
-	 {?__(29,"Save Unused Materials"),save_unused_materials,
-	  ?__(30,"Include unused materials when saving a .wings file"),
-	  save_unused_mats()},
+      %% if there are more options we'll make a panel
+     {?__(29,"Save Unused Materials"),save_unused_materials,
+      ?__(30,"Include unused materials when saving a .wings file"),
+      save_unused_mats()},
      separator,
      {?__(16,"Revert"),revert,
       ?__(17,"Revert current scene to the saved contents")},
@@ -142,13 +157,21 @@ menu(_) ->
      separator,
      {?__(24,"Install Plug-In"),install_plugin,
       ?__(27,"Install a plug-in")},
+     separator,
+     {?__(31,"Save Preference Subset..."),save_pref,
+      ?__(32,"Save a preference subset from your current settings")},
+     {?__(33,"Load Preference Subset"),
+       {load_pref,
+         [{?__(35,"Load..."),custom_theme,
+           ?__(36,"Load a previously saved preference subset")}]
+          ++wings_pref:recent_prefs()}},
      separator|recent_files(Tail)].
 
 save_unused_mats() ->
     case wings_pref:get_value(save_unused_materials) of
 	  true -> [crossmark];
 	  false -> []
-	end.
+    end.
 
 command(new, St) ->
     new(St);
@@ -183,7 +206,7 @@ command(save_incr, St) ->
 command(revert, St0) ->
     case revert(St0) of
 	{error,Reason} ->
-	    wings_u:error(?__(1,"Revert failed: ") ++ Reason),
+	    wings_u:error_msg(?__(1,"Revert failed: ") ++ Reason),
 	    St0;
 	#st{}=St -> {save_state,St}
     end;
@@ -223,6 +246,15 @@ command(install_plugin, _St) ->
     install_plugin();
 command({install_plugin,Filename}, _St) ->
     wings_plugin:install(Filename);
+
+command(save_pref, _St) ->
+    wings_pref:pref(save);
+command({load_pref,Request}, St) ->
+    wings_pref:pref({load,Request,St});
+command({pref,Request}, St) ->
+    wings_pref:pref(Request,St),
+    keep;
+
 command(quit, #st{saved=true}) ->
     quit;
 command(quit, _) ->
@@ -240,7 +272,7 @@ command(Key, St) when is_integer(Key), 1 =< Key ->
 	false ->
 	    Recent = delete_nth(Recent0, Key),
 	    wings_pref:set_value(recent_files, Recent),
-	    wings_u:error(?__(5,"This file has been moved or deleted."))
+	    wings_u:error_msg(?__(5,"This file has been moved or deleted."))
     end.
 
 delete_nth([_|T], 1) -> T;
@@ -254,7 +286,8 @@ confirmed_new(#st{file=File}=St) ->
 
 new(#st{saved=true}=St0) ->
     St1 = clean_st(St0#st{file=undefined}),
-    St = clean_images(wings_undo:init(St1)),
+    St2 = clean_images(wings_undo:init(St1)),
+    St = wings_shape:create_folder_system(St2),
     wings_u:caption(St),
     {new,St#st{saved=true}};
 new(#st{}=St0) ->		      %File is not saved or autosaved.
@@ -293,16 +326,17 @@ confirmed_open(Name, St0) ->
 		  %%   Name: Original name of file to be opened.
 		  %%   File: Either original file or the autosave file
 		  St1 = clean_st(St0#st{file=undefined}),
-		  St2 = wings_undo:init(St1),
+		  St2 = wings_shape:create_folder_system(wings_undo:init(St1)),
 		  case ?SLOW(wings_ff_wings:import(File, St2)) of
 		      #st{}=St3 ->
 			  set_cwd(dirname(File)),
-			  St = clean_images(St3),
+			  St4 = clean_images(St3),
+			  St = wings_shape:recreate_folder_system(St4),
 			  add_recent(Name),
 			  wings_u:caption(St#st{saved=true,file=Name});
 		      {error,Reason} ->
 			  clean_new_images(St2),
-			  wings_u:error(?__(1,"Read failed: ") ++ Reason)
+			  wings_u:error_msg(?__(1,"Read failed: ") ++ Reason)
 		  end
 	  end,
     use_autosave(Name, Fun).
@@ -338,10 +372,10 @@ merge(Name, St0) ->
 		  case ?SLOW(wings_ff_wings:import(File, St0)) of
 		      {error,Reason} ->
 			  clean_new_images(St1),
-			  wings_u:error(?__(2,"Read failed: ") ++ Reason);
+			  wings_u:error_msg(?__(2,"Read failed: ") ++ Reason);
 		      #st{}=St ->
 			  set_cwd(dirname(Name)),
-			  St#st{saved=false}
+			  wings_shape:recreate_folder_system(St)
 		  end
 	  end,
     use_autosave(Name, Fun).
@@ -376,14 +410,14 @@ save_now(Next, #st{file=Name}=St) ->
 	    maybe_send_action(Next),
 	    {saved,wings_u:caption(St#st{saved=true})};
 	{error,Reason} ->
-	    wings_u:error(?__(1,"Save failed: ") ++ Reason)
+	    wings_u:error_msg(?__(1,"Save failed: ") ++ Reason)
     end.
 
 maybe_send_action(ignore) -> keep;
 maybe_send_action(Action) -> wings_wm:later({action,Action}).
     
 save_selected(#st{sel=[]}) ->
-    wings_u:error(?__(1,"This command requires a selection."));
+    wings_u:error_msg(?__(1,"This command requires a selection."));
 save_selected(St) ->
     String = case os:type() of
         {win32,_} -> "Save Selected";
@@ -399,7 +433,7 @@ save_selected(Name, #st{shapes=Shs0,sel=Sel}=St0) ->
     St = St0#st{shapes=gb_trees:from_orddict(Shs)},
     case ?SLOW(wings_ff_wings:export(Name, St)) of
 	ok -> keep;
-	{error,Reason} -> wings_u:error(Reason)
+	{error,Reason} -> wings_u:error_msg(Reason)
     end.
 
 %%%
@@ -578,9 +612,10 @@ recent_files(Tail) ->
 	    recent_files_1(Files, 1, Help, [separator|Tail])
     end.
 
-recent_files_1([{Base0,_}|T], I, Help, Tail) ->
+recent_files_1([{Base0,Base1}|T], I, Help0, Tail) ->
     Base = wings_u:pretty_filename(Base0),
-    [{Base,I,Help}|recent_files_1(T, I+1, Help, Tail)];
+    Help = lists:flatten([Help0," -- "|wings_u:pretty_filename(Base1)]),
+    [{Base,I,Help}|recent_files_1(T, I+1, Help0, Tail)];
 recent_files_1([], _, _, Tail) -> Tail.
 
 %%
@@ -589,9 +624,11 @@ recent_files_1([], _, _, Tail) -> Tail.
 
 revert(#st{file=undefined}=St) -> St;
 revert(#st{file=File}=St0) ->
-    St1 = clean_st(St0),
+    St1 = wings_shape:create_folder_system(clean_st(St0)),
     case ?SLOW(wings_ff_wings:import(File, St1)) of
-	#st{}=St -> clean_images(St);
+	#st{}=St2 ->
+	    St = wings_shape:recreate_folder_system(St2),
+	    clean_images(St);
 	{error,_}=Error ->
 	    Error
     end.
@@ -610,7 +647,7 @@ import_ndo(Name, St0) ->
 	#st{}=St ->
 	    {save_state,St};
 	{error,Reason} ->
-	    wings_u:error(?__(1,"Import failed: ") ++ Reason),
+	    wings_u:error_msg(?__(1,"Import failed: ") ++ Reason),
 	    St0
     end.
 
@@ -624,8 +661,14 @@ import_image(Name) ->
 	Im when is_integer(Im) ->
 	    keep;
 	{error,Error} ->
-	    wings_u:error(?__(1,"Failed to load \"~s\": ~s\n"),
-			     [Name,file:format_error(Error)])
+		case Error of
+		100902 -> % GLU_OUT_OF_MEMORY
+			wings_u:error_msg(?__(2,"The image cannot be loaded.~nFile: \"~s\"~n GLU Error: ~p - ~s~n"),
+				  	[Name,Error, glu:errorString(Error)]);
+		_ ->
+	    	wings_u:error_msg(?__(1,"Failed to load \"~s\": ~s\n"),
+				     [Name,file:format_error(Error)])
+		end
     end.
 
 %%
@@ -640,7 +683,7 @@ export_ndo(Cmd, Title, St) ->
 do_export_ndo(Name, St) ->
     case wings_ff_ndo:export(Name, St) of
 	ok -> keep;
-	{error,Reason} -> wings_u:error(Reason)
+	{error,Reason} -> wings_u:error_msg(Reason)
     end.
 
 %%%
