@@ -3,7 +3,7 @@
 %%
 %%     A few OpenGL utilities.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -148,7 +148,7 @@ check_error(Mod, Line) ->
 	    ok;
 	Str ->
 	    io:format("~p, line ~p: ~s\n", [Mod,Line,Str]),
-	    erlang:error(gl_error, [Mod,Line])
+	    error(gl_error, [Mod,Line])
     end.
 -else.
 check_error(_Mod, _Line) ->
