@@ -3,7 +3,7 @@
 %%
 %%     This module manages the face materials (i.e. colors and textures).
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -188,15 +188,17 @@ select_material(Mat, #st{shapes=Shs}=St) ->
 		end, [], gb_trees:values(Shs)),
     wings_sel:set(Sel, St).
 
-select_material_1(#we{id=Id,fs=Ftab}=We, Mat, Acc) ->
+select_material_1(#we{id=Id,fs=Ftab,perm=Perm}=We, Mat, Acc) when ?IS_SELECTABLE(Perm) ->
     MatFaces = wings_facemat:mat_faces(gb_trees:to_list(Ftab), We),
     case keyfind(Mat, 1, MatFaces) of
 	false ->
 	    Acc;
 	{Mat,FaceInfoList} ->
-	    Sel = [F || {F,_} <- FaceInfoList],
+	    Sel = [F || {F,_} <- FaceInfoList, F >= 0],
 	    [{Id,gb_sets:from_ordset(Sel)}|Acc]
-    end.
+    end;
+select_material_1(_, _, Acc) ->
+    Acc.
 
 set_material(Mat, #st{selmode=face}=St) ->
     wings_sel:map(fun(Faces, We) ->
@@ -650,7 +652,7 @@ plugin_results(Name, Mat0, Res0) ->
 	{_,Res} ->
 	    io:format(?__(1,"Material editor plugin(s) left garbage:~n    ~P~n"), 
 		      [Res,20]),
-		      wings_u:error(?__(2,"Plugin(s) left garbage"))
+		      wings_u:error_msg(?__(2,"Plugin(s) left garbage"))
     end.
 
 update_maps(Mat0, More0) ->
