@@ -3,7 +3,7 @@
 %%
 %%     Menu utilities and helpers.
 %%
-%%  Copyright (c) 2002-2009 Bjorn Gustavsson
+%%  Copyright (c) 2002-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -12,7 +12,8 @@
 %%
 
 -module(wings_menu_util).
--export([directions/1,directions/2,scale/1,rotate/1,flatten/0,all_xyz/0]).
+-export([directions/1,directions/2,scale/1,rotate/1,flatten/0,all_xyz/0,
+	 crossmark/1]).
 
 -include("wings.hrl").
 
@@ -201,8 +202,8 @@ flatten(1, [flatten,vertex]) ->
     %% Vertex mode flatten.
     flatten_common();
 flatten(1, [flatten,edge]) ->
-    %% Vertex mode flatten.
-    flatten_common();
+    %% Edge mode flatten.
+    [flatten_common()|flatten_edge_loops()];
 flatten(1, _) ->
     %% Face mode flatten.
     [flatten_fun(normal)|flatten_common()];
@@ -234,6 +235,11 @@ flatten_fun_1(Vec, Axis, String) ->
     Help = {Help0,[],?STR(flatten_fun_1,1,"Pick point on plane")},
     {String,F,Help,[]}.
 
+flatten_edge_loops() ->
+    [separator,
+     {?__(1,"Edge Loops"),edge_loop,
+      ?__(2,"Flatten each closed edge loop to its normal")}].
+
 %%%
 %%% General directions.
 %%%
@@ -325,9 +331,9 @@ dir_help_1([lift|_], [free|Text]) ->
 dir_help_1([duplicate|_], [free|Text]) ->
     ?STR(dir_help_1,12,"Duplicate and move freely ")++ Text;
 dir_help_1([shell_extrude|_], [normal|_]) ->
-    ?STR(dir_help_1,24,"Extract and Extrude faces as region, then move faces along the region's normal");
+    ?STR(dir_help_1,27,"Extract and Extrude faces as region, then move faces along the region's normal");
 dir_help_1([shell_extrude|_], [free|Text]) ->
-    ?STR(dir_help_1,25,"Extract and Extrude faces as region, then move faces ") ++ Text;
+    ?STR(dir_help_1,28,"Extract and Extrude faces as region, then move faces ") ++ Text;
 
 %% Axis
 dir_help_1([move|_], Text) ->
@@ -357,3 +363,16 @@ dir_help_1([duplicate|_], Text) ->
 dir_help_1([shell_extrude|_], Text) ->
     ?STR(dir_help_1,26,"Extract and Extrude faces as region, then move along ") ++ Text;
 dir_help_1(_, _) -> "".
+
+%% Menu checkmark
+crossmark(Key) ->
+    Val = case wings_pref:get_value(Key) of
+	      undefined ->
+		  {_,Client} = wings_wm:this(),
+		  wings_wm:get_prop(Client, Key);
+	      Other -> Other
+	  end,
+    case Val of
+	false -> [];
+	true -> [crossmark]
+    end.
