@@ -3,7 +3,7 @@
 %%
 %%     Maintains the outliner window.
 %%
-%%  Copyright (c) 2003-2008 Bjorn Gustavsson
+%%  Copyright (c) 2003-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -12,7 +12,7 @@
 %%
 
 -module(wings_outliner).
--export([window/1,window/3]).
+-export([window/1,window/4]).
 
 -define(NEED_ESDL, 1).
 -define(NEED_OPENGL, 1).
@@ -45,18 +45,18 @@ window(St) ->
 	    W = 28*?CHAR_WIDTH,
 	    Pos = {DeskW-5,DeskY+55},
 	    Size = {W,DeskH div 2},
-	    window(Pos, Size, St),
+	    window(Pos, Size, [], St),
 	    keep
     end.
 
-window(Pos, Size, St) ->
+window(Pos, Size, Ps, St) ->
     Ost = #ost{first=0,lh=18,active=-1},
     Current = {current_state,St},
     Op = {seq,push,event(Current, Ost)},
     Props = [{display_lists,geom_display_lists}],
     wings_wm:toplevel(outliner, title(), Pos, Size,
 		      [{sizeable,?PANE_COLOR},closable,vscroller,{anchor,ne},
-		       {properties,Props}], Op),
+		       {properties,Props}|Ps], Op),
     F = fun({image,_,_}) -> yes;
 	   (_) -> no
 	end,
@@ -68,7 +68,14 @@ get_event(Ost) ->
 event(redraw, Ost) ->
     wings_io:ortho_setup(),
     {W,H} = wings_wm:win_size(),
-    wings_io:border(0, 0, W-1, H-1, ?PANE_COLOR),
+    case wings_pref:get_value(bitmap_icons) of
+        false -> wings_io:border(0, 0, W-1, H-1, ?PANE_COLOR);
+        true ->
+          wings_io:blend(wings_pref:get_value(outliner_geograph_bg),
+            fun(Color) ->
+              wings_io:border(0, 0, W-1, H-1, Color)
+            end)
+    end,
     draw_objects(Ost),
     keep;
 event(resized, Ost) ->
@@ -243,7 +250,7 @@ image_menu(Id, Im) ->
 image_menu_1(Id, #e3d_image{filename=none}) ->
     [{?__(1,"Make External..."),menu_cmd(make_external, Id)}|common_image_menu(Id)];
 image_menu_1(Id, _) ->
-    [{?__(2,"Refresh"),menu_cmd(refresh_image, Id)},
+    [{?__(2,"Refresh"),menu_cmd(refresh_image, Id),?__(11,"Update image to the contents of the saved file")},
      {?__(3,"Make Internal"),menu_cmd(make_internal, Id)}|common_image_menu(Id)].
 
 common_image_menu(Id) ->
@@ -371,6 +378,7 @@ make_external(Id) ->
 		   case wings_image:image_write(Ps) of
 		       ok ->
 			   wings_image:update_filename(Id, Name),
+			   rename(Id, Name),
 			   keep;
 		       {_,Error0} ->
 			   Error = Name ++ ": " ++ file:format_error(Error0),
@@ -405,7 +413,9 @@ export_image(Id) ->
 		   Image = wings_image:info(Id),
 		   Ps = [{image,Image},{filename,Name}],
 		   case wings_image:image_write(Ps) of
-		       ok -> keep;
+		       ok ->
+			   rename(Id, Name),
+			   keep;
 		       {_,Error0} ->
 			   Error = Name ++ ": " ++ file:format_error(Error0),
 			   wings_u:message(Error)
@@ -443,22 +453,14 @@ update_state(St, #ost{first=OldFirst}=Ost0) ->
 	    Ost#ost{first=First}
     end.
 
-% update_state_1(#st{shapes=Shs,mat=Mat}=St, #ost{st=#st{shapes=Shs,mat=Mat}}=Ost) ->
-%     Ost#ost{st=St};
 update_state_1(St, Ost) ->
     update_state_2(St, Ost).
 
 update_state_2(#st{mat=Mat,shapes=Shs0}=St, #ost{os=Objs0,active=Act0}=Ost) ->
-    Objects =
-      case wings_pref:get_value(objects_in_outliner) of
-        true -> [{object,Id,Name} || #we{id=Id,name=Name}=We <- gb_trees:values(Shs0),
-				     not ?IS_ANY_LIGHT(We)];
-        _ -> []
-      end,
     Lights = [{light,Id,Name} || #we{id=Id,name=Name}=We <- gb_trees:values(Shs0),
 			         ?IS_ANY_LIGHT(We)],
     Materials = [make_mat(M) || M <- gb_trees:to_list(Mat)],
-    Objs = Objects ++ Lights ++ Materials ++ update_images(),
+    Objs = Lights ++ Materials ++ update_images(),
     case Objs of
 	Objs0 -> ok;
 	_ -> wings_wm:dirty()
@@ -540,16 +542,17 @@ draw_objects(#ost{os=Objs0,first=First,lh=Lh,active=Active,n=N0}=Ost) ->
 	    N1 when N1 < Lines -> N1;
 	    _ -> Lines
 	end,
-    draw_icons(N, Objs, Ost, Lh-2),
-    draw_objects_1(N, Objs, Ost, R, Active-First, Lh-2).
+	B = wings_pref:get_value(bitmap_icons),
+    draw_icons(N, B, Objs, Ost, Lh-2),
+    draw_objects_1(N, B, Objs, Ost, R, Active-First, Lh-2).
 
-draw_objects_1(0, _, _, _, _, _) -> ok;
-draw_objects_1(N, [O|Objs], #ost{lh=Lh}=Ost, R, Active, Y) ->
+draw_objects_1(0, _, _, _, _, _, _) -> ok;
+draw_objects_1(N, B, [O|Objs], #ost{lh=Lh}=Ost, R, Active, Y) ->
     case O of
 	{material,Name,Color,TextColor} ->
-	    wings_io:border(2, Y-10, 12, 12, Color),
+	    wings_io:border(4, Y-11, 12, 12, Color),
 	    gl:color3fv(TextColor),
-	    gl:rasterPos2f(5.5, Y),
+	    gl:rasterPos2f(7.5, Y-1),
 	    wings_io:draw_char(m_bitmap()),
 	    gl:color3b(0, 0, 0);
 	{image,_,#e3d_image{name=Name}} -> ok;
@@ -557,25 +560,36 @@ draw_objects_1(N, [O|Objs], #ost{lh=Lh}=Ost, R, Active, Y) ->
 	{image_preview,_} -> Name = [];
 	ignore -> Name = []
     end,
-    if
-	Active == 0 ->
+    case Active =:= 0 of
+	true when B ->
+	    gl:color3fv(wings_pref:get_value(outliner_geograph_hl)),
+	    gl:recti(name_pos()-2, Y-?CHAR_HEIGHT, R-2, Y+4),
+	    gl:color3fv(wings_pref:get_value(outliner_geograph_hl_text));
+	true ->
 	    gl:color3f(0, 0, 0.5),
 	    gl:recti(name_pos()-2, Y-?CHAR_HEIGHT, R-2, Y+4),
 	    gl:color3f(1, 1, 1);
-	true -> ok
+	false when B ->
+	    gl:color3fv(wings_pref:get_value(outliner_geograph_text));
+	false -> ok
     end,
     wings_io:text_at(name_pos(), Y, Name),
     gl:color3b(0, 0, 0),
-    draw_objects_1(N-1, Objs, Ost, R, Active-1, Y+Lh).
+    draw_objects_1(N-1, B, Objs, Ost, R, Active-1, Y+Lh).
 
-draw_icons(N, Objs, Ost, Y) ->
-    wings_io:draw_icons(fun() -> draw_icons_1(N, Objs, Ost, Y-14) end),
+draw_icons(N, B, Objs, Ost, Y) ->
+    case B of
+      false ->
+        wings_io:draw_icons(fun() -> draw_icons_1(N, Objs, Ost, Y-14) end);
+      true ->
+        wings_io:draw_icons(fun() -> draw_bitmap_icons(N, Objs, Ost, Y) end)
+    end,
     gl:enable(?GL_TEXTURE_2D),
     gl:texEnvi(?GL_TEXTURE_ENV, ?GL_TEXTURE_ENV_MODE, ?GL_REPLACE),
     draw_previews(N, Objs, Ost, Y-14),
     gl:bindTexture(?GL_TEXTURE_2D, 0),
     gl:disable(?GL_TEXTURE_2D).
-    
+
 draw_icons_1(0, _, _, _) -> ok;
 draw_icons_1(N, [ignore|Objs], #ost{lh=Lh}=Ost, Y) ->
     draw_icons_1(N-1, Objs, Ost, Y+Lh);
@@ -599,6 +613,50 @@ draw_icons_1(N, [O|Objs], #ost{lh=Lh}=Ost, Y) ->
     end,
     draw_icons_1(N-1, Objs, Ost, Y+Lh).
 
+draw_bitmap_icons(0, _, _, _) -> ok;
+draw_bitmap_icons(N, [ignore|Objs], #ost{lh=Lh}=Ost, Y) ->
+    draw_bitmap_icons(N-1, Objs, Ost, Y+Lh);
+draw_bitmap_icons(N, [O|Objs], #ost{lh=Lh}=Ost, Y0) ->
+    X = 2,
+    Y = Y0+2,
+    Type = element(1, O),
+    case Type of
+	object ->
+	    Cube = wings_shape:cube_bitmap(),
+	    wings_shape:draw_bitmap_16(X, Y, Cube),
+	    gl:color3fv(wings_pref:get_value(outliner_geograph_disabled)),
+	    ShadeCube = wings_shape:selcube_bitmap(),
+	    wings_shape:draw_bitmap_16(X, Y, ShadeCube);
+	light ->
+	    gl:color3fv(wings_pref:get_value(outliner_geograph_text)),
+	    Light = wings_shape:light_bitmap_0(),
+	    wings_shape:draw_bitmap_16(X, Y, Light),
+	    gl:color3fv({1.0, 1.0, 0.5}),
+	    LightObj = wings_shape:light_bitmap_1(),
+	    wings_shape:draw_bitmap_16(X, Y, LightObj);
+	image ->
+	    case O of
+	      {_,_,#e3d_image{filename=none}} ->
+	        gl:color3fv(wings_pref:get_value(outliner_geograph_text)),
+	        ImgObj0 = img_bitmap_0(),
+	        wings_shape:draw_bitmap_16(X, Y, ImgObj0),
+	        gl:color3fv(wings_pref:get_value(outliner_geograph_disabled)),
+	        ImgObj1 = img_bitmap_1(),
+	        wings_shape:draw_bitmap_16(X, Y, ImgObj1);
+	      _ ->
+	        gl:color3fv(wings_pref:get_value(outliner_geograph_disabled)),
+	        ImgObj0 = img_bitmap_0(),
+	        wings_shape:draw_bitmap_16(X, Y, ImgObj0),
+	        gl:color3fv(wings_pref:get_value(outliner_geograph_text)),
+	        ImgObj1 = img_bitmap_1(),
+	        wings_shape:draw_bitmap_16(X, Y, ImgObj1)
+	    end;
+	image_preview -> ok;
+	material -> ok
+    end,
+    gl:color3b(0, 0, 0),
+    draw_bitmap_icons(N-1, Objs, Ost, Y0 + Lh).
+
 draw_previews(0, _, _, _) -> ok;
 draw_previews(N, [{image_preview,Im}|Objs], #ost{lh=Lh}=Ost, Y) ->
     W = H = 2*Lh,
@@ -618,6 +676,43 @@ m_bitmap() ->
        2#11000110,
        2#10000010>>}.
 
+img_bitmap_0() ->
+     <<2#0000000000000000:16,
+       2#0000000000000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000000000000:16,
+       2#0000000000000000:16>>.
+
+img_bitmap_1() ->
+     <<2#0000000000000000:16,
+       2#0000000000000000:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0000000011111100:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0011111100000000:16,
+       2#0000000000000000:16,
+       2#0000000000000000:16>>.
+
+
 top_of_first_object() ->
     0.
 
@@ -634,3 +729,7 @@ lines(#ost{lh=Lh}) ->
 
 title() ->
     ?__(1,"Outliner").
+
+rename(Id, File) ->
+    Name = filename:basename(File),
+    wings_image:rename(Id, Name).
