@@ -3,7 +3,7 @@
 %%
 %%     This module contains a progress bar
 %%
-%%  Copyright (c) 2004-2009 Dan Gudmundsson and Bjorn Gustavsson 
+%%  Copyright (c) 2004-2011 Dan Gudmundsson and Bjorn Gustavsson 
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -25,8 +25,13 @@
 -define(PB, progress_bar).
 
 start(Msg) when is_list(Msg) ->
-    WinInfo = wings_wm:viewport(message),
-    cast({start,Msg,percent,WinInfo}).
+    case get(wings_not_running) of
+	undefined ->
+	    WinInfo = wings_wm:viewport(message),
+	    cast({start,Msg,percent,WinInfo});
+	_ ->
+	    ignore
+    end.
 
 update(Percent) when is_float(Percent) -> 
     cast({update,"",Percent}).
@@ -97,23 +102,17 @@ reply(Pid, What) ->
 
 %% Start progressbar process
 init() ->
-    case wings_pref:get_value(no_progress_bar) of
-	true ->
-	    cast(terminate),
-	    sent_termination_request;
-	false ->
-	    case whereis(?PB) of
-		undefined ->
-		    POpt = wings_io:get_process_option(),
-		    Pid = spawn_link(fun() ->
-					     wings_io:set_process_option(POpt),
-					     loop(#state{})
-				     end),
+    case whereis(?PB) of
+	undefined ->
+	    POpt = wings_io:get_process_option(),
+	    Pid = spawn_link(fun() ->
+				     wings_io:set_process_option(POpt),
+				     loop(#state{})
+			     end),
 
-		    register(?PB, Pid),
-		    started;
-		_ -> already_started
-	    end
+	    register(?PB, Pid),
+	    started;
+	_ -> already_started
     end.
 
 loop(#state{refresh=After,level=Level,msg=Msg0}=S0) ->
