@@ -1,19 +1,19 @@
-%%		       -*- mode:erlang; coding:iso-latin-1-unix -*-
+%%		       -*- mode:erlang; encoding: utf-8 -*-
 %%
 %%  wings_pref_dlg.erl --
 %%
 %%     Preference management.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id$
 %%
 
 -module(wings_pref_dlg).
 -export([menu/1,command/2]).
+-export([set_values/2]).
 
 -include("wings.hrl").
 -import(wings_pref, [get_value/1,set_value/2]).
@@ -83,20 +83,16 @@ gen_prefs() ->
       {hframe,
        [{vframe,
 	 [{hframe,
-	   [{?__(15,"Vertices"),vertex_hilite},
-	    {?__(16,"Edges"),edge_hilite},
-	    {?__(17,"Faces"),face_hilite},
-	    {?__(18,"Objects"),body_hilite}]},
+	   [{?__(18,"Objects"),body_hilite}]},
 	  {hframe,
 	   [{label,?__(19,"Unselected")},{color,unselected_hlite},
 	    {label,?__(20,"Selected")},{color,selected_hlite}]},
-	  {?__(21,"Smart Highlighting"),smart_highlighting}],
+	  {?__(71,"Conditional Deselection"), conditional_deselect,
+	   [{info,?__(72,"Deselecting keeps you in the same Selection Mode, unless there is no selection")}]}],
 	 [{title,?__(22,"Highlighting")}]},
 
 	{vframe,
-	 [{vradio,[{?__(23,"Solid Face Selections"),solid},
-               {?__(24,"Stippled Face Selections"),stippled}], selection_style},
-	  {hframe,[{label,?__(25,"Selection Color")},{color,selected_color}]}],
+	 [{label,?__(25,"Selection Color")},{color,selected_color}],
 	 [{title,?__(26,"Selection")}]}]},
 
       {hframe,
@@ -108,9 +104,7 @@ gen_prefs() ->
 	     [{info,?__(30,"Color of background for information text (including transparency)")}]}]},
 	  {hframe,
 	     [{?__(61,"Verbose"),info_enhanced_text,
-	       [{info,?__(62,"Show additional information about selected elements")}]},
-	     {?__(63,"Mouseover"),info_text_on_hilite,
-	       [{info,?__(64,"Show additional information for highlighted element on mouseover")}]}]}],
+	       [{info,?__(62,"Show additional information about selected elements")}]}]}],
 	 [{title,?__(31,"Information Text")}]},
 	{vframe,
 	 [{label_column,
@@ -128,8 +122,7 @@ gen_prefs() ->
 	 [{title,?__(55,"Materials")}]}]},
 	  {hframe,
 	   [{vframe,
-	 [{?__(36,"Show Axis Letters"),show_axis_letters},
-	  {?__(49,"Constrain Axes to Grid"),constrain_axes},
+	 [{?__(49,"Constrain Axes to Grid"),constrain_axes},
 	  {?__(56,"Show Mini Axis"),mini_axis}]},
 	  {hframe,
 	   [{label_column,
@@ -162,84 +155,47 @@ multisampling() ->
 	      [{title,"Anti-Aliasing"}]}]
     end.
 
-
 advanced_prefs() ->
-%%     DisableHook = fun (is_disabled, {_Var,_I,Store}) ->
-%% 			  not gb_trees:get(advanced_menus, Store);
-%% 		      (_, _) -> void
-%% 		  end,
-    SuperDisable = fun (is_disabled, {_Var,_I,Store}) ->
-%%			   not gb_trees:get(advanced_menus, Store) orelse
-			       not gb_trees:get(use_temp_sel, Store);
-		       (_, _) ->	void
-		   end,
-    HighlightDisable = fun (is_disabled, {_Var,_I,Store}) ->
-		       not gb_trees:get(use_temp_sel, Store);
-		       (_, _) ->	void
-		   end,
-%%    Flags = [{hook,DisableHook}],
-    Flags = [],
     Disable = fun (is_disabled, {_Var,_I, Store}) ->
 		       not gb_trees:get(drag_custom,Store);
 		       (_, _) -> void
 		       end,
-
     {hframe,
     [{vframe,
      [{?__(1,"Default Commands"),default_commands,
        [{info,?__(2,"Allow defining commands that can be invoked by Ctrl+L or Ctrl+M")}]},
-      {?__(3,"Use Highlight as Temporary Selection"),use_temp_sel,
-       [{info,?__(4,"If there is no selection, ")++
-      ?__(5,"allow commands to act on the highlighted element")}]},
       {?__(6,"Hide Selection While Dragging"),hide_sel_while_dragging,
        [{info,?__(7,"Don't show the selection in any interactive command")}]},
       {?__(8,"Hide Selection While Moving Camera"),hide_sel_in_camera_moves,
        [{info,?__(9,"Don't show the selection when the camera is being moved")}]},
-      panel,
-%%      {?__(10,"Advanced Menus"),advanced_menus,
-%%       [{info,?__(11,"More commands and more options, such as magnets")}]},
-      {?__(21,"Use Mirror for Selections"),use_mirror_for_sels,
-       [{info,?__(22,"Default to using the virtual mirror for secondary selections")}]},
-      {?__(12,"Power-user temporary selections"),use_super_temp_sel,
-       [{info,?__(13,"In the secondary selection mode, RMB-clicking always adds to the selection")},{hook,SuperDisable}]},
       {?__(54,"Extend selection via hotkey on temporary highlight"),hilite_select,
-	   [{info,?__(55,"Affects: Select All, Edge Loop, Edge Ring, Select Similar, Similar Normals, Similar Materials, and Similar Area.")},
-	     {hook,HighlightDisable}]},
-      {?__(53,"Launch Wings in Tweak Mode"),start_in_tweak},
+	   [{info,?__(55,"Affects: Select All, Edge Loop, Edge Ring, Select Similar, Similar Normals, Similar Materials, and Similar Area.")}]},
      {vframe,
        [{label_column,
      [{?__(14,"Length"),active_vector_size,
-       [{info,?__(15,"Length of vector in secondary selections")},{range,{0.1,10.0}}|Flags]},
+       [{info,?__(15,"Length of vector in secondary selections")},{range,{0.1,10.0}}]},
       {?__(16,"Width"),active_vector_width,
-       [{info,?__(17,"Width of vector (in pixels)")},{range,{1.0,10.0}}|Flags]},
+       [{info,?__(17,"Width of vector (in pixels)")},{range,{1.0,10.0}}]},
       {color,?__(18,"Color"),active_vector_color,
-       [{info,?__(19,"Color of vector")}|Flags]}]}],
+       [{info,?__(19,"Color of vector")}]}]}],
        [{title,?__(20,"Vector Display")}]},
       {vframe,
        [{label_column,
      [{?__(23,"Size"),clip_plane_size,
-       [{info,?__(24,"Size of user-defined clipping indicator")},{range,{0.1,100.0}}|Flags]},
+       [{info,?__(24,"Size of user-defined clipping indicator")},{range,{0.1,100.0}}]},
       {color,?__(25,"Color"),clip_plane_color,
        [{info,?__(26,"Color of user-defined clipping indicator")}]}]}],
-       [{title,?__(27,"Clipping Plane Indicator")}]},
+       [{title,?__(27,"Clipping Plane Indicator")}]}]},
      {vframe,
-       [{?__(28,"Selected Geometry"),highlight_aim_at_selected},
-        {?__(29,"Unselected Geometry"),highlight_aim_at_unselected}],
-       [{title,?__(30,"Highlight Aim Targets")},{hook,HighlightDisable}]}]},
-     {vframe,
-        [{vframe,
-          [{?__(36,"Switching Modes Resets Model's Position"),drag_resets,
-              [{info,?__(37,"Only affects tools with more than one drag mode such as Edge|Slide")}]},
-           {?__(38,"Customize Drag Response"),drag_custom,
-               [{info,?__(39,"Customize the drag to mouse motion ratio. Unchecked uses default settings.")}]}]},
+      [{vframe,
+       [{?__(38,"Customize Drag Response"),drag_custom,
+         [{info,?__(39,"Customize the drag to mouse motion ratio. Unchecked uses default settings.")}]}]},
         {vframe,
-          [{?__(40,"Factor in Distance from Camera"),drag_cam_dist_abs},
-           {hframe,
+          [{hframe,
               [{slider,{text,drag_speed_abs,[{range,{1.0,10.0}}]}}]}],
           [{title,?__(50,"Mouse Speed for Dragging")},{hook,Disable}]},
          {vframe,
-          [{?__(43,"Factor in Distance from Camera"),drag_cam_dist_relative},
-            {hframe,
+          [{hframe,
               [{slider,{text,drag_speed_relative,[{range,{1.0,10.0}}]}}]}],
            [{title,?__(51,"Mouse Speed for Scaling")},{hook,Disable}]},
          {vframe,
@@ -416,49 +372,65 @@ ui_prefs() ->
 		      {?__(52,"Blue Cube"), bluecube},
 		      {?__(53,"Purple Tube"), purpletube}],
     Langs = [{language_name(L),L} || L <- Langs0],
+
+    Bitmap = fun (is_disabled, {_Var,_I,Store}) ->
+		       not gb_trees:get(bitmap_icons, Store);
+		       (_, _) ->	void
+		   end,
+
     {vframe,
      [{hframe,
        [{vframe,
-	 [{vframe,
-	   [{menu,Fonts,new_system_font}],
-	   [{title,?__(1,"System Font")}]},
-	  {hframe,
+	 [{hframe,
 	   [{vframe,
 	     [{label,?__(12,"Title (Active) Background")},
 	      {label,?__(11,"Title (Passive) Background")},
 	      {label,?__(10,"Title Text")},
+	       separator,
 	      {label,?__(2,"Desktop/Geometry Background")},
+	       separator,
 	      {label,?__(28,"Menubar Background")},
 	      {label,?__(31,"Menubar Text")},
+	       separator,
 	      {label,?__(6,"Menu Background")},
 	      {label,?__(3,"Menu Text")},
 	      {label,?__(4,"Menu Highlight")},
 	      {label,?__(5,"Menu Highlighted Text")},
+	       separator,
 	      {label,?__(9,"Dialog Background")},
 	      {label,?__(7,"Dialog Text")},
 	      {label,?__(8,"Dialog (Disabled) Text")},
+	       separator,
 	      {label,?__(29,"Info Line Background")},
 	      {label,?__(30,"Info Line Text")}]},
 	    {vframe,
 	     [{color,title_active_color},
 	      {color,title_passive_color},
 	      {color,title_text_color},
+	        separator,
 	      {color,background_color},
+	       separator,
 	      {color,menu_bar_bg},
 	      {color,menubar_text},
+	       separator,
 	      {color,menu_color},
 	      {color,menu_text},
 	      {color,menu_hilite},
 	      {color,menu_hilited_text},
+	       separator,
 	      {color,dialog_color},
 	      {color,dialog_text},
 	      {color,dialog_disabled},
+	       separator,
 	      {color,info_line_bg},
 	      {color,info_line_text}]}],
 	   [{title,?__(13,"Colors")}]}
 	 ]},
 	{vframe,
 	 [{vframe,
+	   [{menu,Fonts,new_system_font}],
+	   [{title,?__(1,"System Font")}]},
+	  {vframe,
 	   [{menu,Langs,language}],
 	   [{title,?__(23,"Language")}]},
 	  {vframe,
@@ -476,17 +448,40 @@ ui_prefs() ->
 			    {color,console_color},
 			    {color,console_text_color},
 			    {color,console_cursor_color}]}],
-	   [{title,?__(22,"Console")}]},
-	  {vframe,
-	   [{menu,InterfaceIcons,interface_icons}],
-	   [{title,?__(50,"Interface Icons")}]}]}
-       ]},
+	   [{title,?__(22,"Console")}]}]},
+    {vframe,
+     [{vframe,
+       [{?__(32,"Adaptive Icons"), bitmap_icons,
+	 [{info, ?__(33,"Icons in the Outliner and Geometry Graph that allow for further color theming")}]},
       {hframe,
-	    [{vframe,[{?__(14,"No Progress Bar"),no_progress_bar},
-	       {?__(25,"View image after rendering"),render_load_image},
-	       {?__(26,"Load image after rendering"),render_iload_image}]},
-	     {vframe,[{?__(24,"Objects in Outliner"),objects_in_outliner},
-	       {?__(27,"Extended Toolbar Icons"),extended_toolbar}]}]},
+	   [{vframe,
+	    [{label,?__(35,"Background")},
+	     {label,?__(36,"Text")},
+	     {label,?__(37,"Highlighter")},
+	     {label,?__(38,"Highlighted Text")},
+	     {label,?__(39,"Disabled")}]},
+	    {vframe,
+	     [{color, outliner_geograph_bg},
+	      {color, outliner_geograph_text},
+	      {color, outliner_geograph_hl},
+	      {color, outliner_geograph_hl_text},
+	      {color, outliner_geograph_disabled}]}],
+	    [{title,?__(13,"Colors")},{hook,Bitmap}]}],
+	    [{title,?__(34,"Outliner/Geometry Graph")}]},
+	  {vframe,
+	   [{menu,InterfaceIcons,interface_icons},
+	    {?__(27,"Extended Toolbar Icons"),extended_toolbar},
+	    {vframe,
+	      [{?__(46,"Show Menu Toolbar"),menu_toolbar,
+	       [{info,?__(41,"Adds a mini toolbar to the tops of the right-click menus")}]},
+	       {menu,[{?__(44,"Large Icons"),big},{?__(45,"Small Icons"),small}],menu_toolbar_size},
+	       {?__(47,"Snap mouse"),menu_toolbar_snap_cursor}],
+	       [{title,?__(40,"Menu Toolbar")}]}],
+	   [{title,?__(50,"Interface Icons")}]}]}
+	   ]},
+      {hframe,
+        [{vframe,
+          [{?__(42,"View render with external viewer"),render_load_image}]}]},
       {oframe,
        [{atom_to_list(Format),viewer_prefs(Format)}
 	|| {Format,_,_} <- wings_job:render_formats()],
@@ -506,24 +501,24 @@ viewer_prefs(Format) ->
 	  {label,?__(3,"..filename..")},
 	  {text,{viewer_postopts,Format},[{width,10}]}]}]}]}.
 
-
-language_name("cs") ->
-    %% "Czech" in Czech as Unicode:
-    [16#10C|"esk"];
+%% Localised languages in their local language with unicode integers
+language_name("cs") -> [268,"esk",253]; %Czech 
 language_name("en") -> "English";
 language_name("de") -> "Deutsch";
-language_name("es") -> "Espaol";
+language_name("es") -> ["Espa",241,"ol"];
 language_name("fi") -> "Suomi";
-language_name("fr") -> "Franais";
+language_name("fr") -> ["Fran",231,"ais"];
 language_name("hu") -> "Hungarian";
 language_name("it") -> "Italiano";
 language_name("pl") -> "Polski";
-language_name("pt") -> "Portugus";
-language_name("tr") -> "Trke";
-language_name("ru") ->
-    %% "Russian" in Russian as Unicode:
-    [1056,1091,1089,1089,1082,1080,1081];
+language_name("pt") -> ["Portugu",234,"s"]; 
+language_name("tr") -> ["T",252,"rk",231,"e"]; %Turkish
+language_name("ru") -> [1056,1091,1089,1089,1082,1080,1081]; %Russian
 language_name("sv") -> "Svenska";
+language_name("ko") -> "Korean";
+language_name("zh-cn") -> "Simplified Chinese";
+language_name("zh-tw") -> "Traditional Chinese";
+language_name("jp") -> "Japanese";
 language_name(Other) -> Other.
 
 misc_prefs() ->
@@ -567,18 +562,16 @@ misc_prefs() ->
        [{title,?__(14,"Proxy Mode")}]},
       {vframe,
        [{hframe,
-	 %% Note: text_display_lists is specially handled
-	 %% in make_query/1 and smart_set_value/3 to have its value inverted
-	 %% to keep preference files backward compatible.
-	 workaround([{text_display_lists,
-		      ?__(15,"Text in menus and dialogs disappear"),
-		      ?__(16,"Problem occurs on some Matrox cards")},
+	 workaround([
 		     {dummy_axis_letter,
 		      ?__(17,"Wings crashes if axes are turned off"),
 		      ?__(18,"Problem occurs on some Matrox cards")},
 		     {jumpy_camera,
 		      ?__(19,"Camera moves and interactive commands are jumpy"),
-		      ?__(20,"Problem occurs on Mac OS X 10.3 (Panther)")}
+		      ?__(20,"Problem occurs on Mac OS X 10.3 (Panther)")},
+		     {ungrab_bug,
+		      ?__(26,"Camera moves steals focus"),
+		      ?__(27,"Problem occurs on linux")}
 		    ])},
 	separator,
 	{hframe,[{label,?__(23,"Edge offsets:")},
@@ -588,7 +581,13 @@ misc_prefs() ->
        [{title,?__(21,"Workarounds")}]},
       {vframe,
        [{hframe,[{?__(24,"Show Develop menu"),show_develop_menu,
-		  [{info,?__(25,"Show a menu with tools for the Wings developers")}]}]}]}
+		  [{info,?__(25,"Show a menu with tools for the Wings developers")}]}]},
+	{vframe,
+	  [{label_column,
+	    [{?__(28,"Maximum menu height in pixels"),max_menu_height,
+	     [{info,?__(29,"Menus are clipped and continue in 'More...' submenu.")
+	       ++" "++?__(30,"Less than 1 sets menu clipping to auto.")}]}]}]}]}
+       
      ]}.
 
 workaround(L) ->
@@ -604,9 +603,6 @@ workaround_1([], A, B) ->
     [{vframe,[{label,?__(1,"Problem")},separator|reverse(A)]},
      {vframe,[{label,?__(2,"Use Workaround?")},separator|reverse(B)]}].
 
-smart_set_value(text_display_lists=Key, Val, St) ->
-    %% Reverse sense to keep backwards comptibility of preferences.
-    smart_set_value_1(Key, not Val, St);
 smart_set_value(Key, Val, St) ->
     smart_set_value_1(Key, Val, St).
 
@@ -636,15 +632,28 @@ smart_set_value_1(Key, Val, St) ->
 		    delayed_set_value(Key, OldVal, Val),
 		    wings_u:message(?__(2,"The change to the interface icons will take\neffect the next time Wings 3D is started."));
 		new_console_font ->
-		    wings_console:window();
+		    delayed_set_value(Key, OldVal, Val),
+		    wings_u:message(?__(4,"The change to the console font will take\neffect the next time Wings 3D is started."));
+		console_color ->
+		    set_console();
+		console_text_color ->
+		    set_console();
+		console_cursor_color ->
+		    set_console();
 		camera_mode ->
 		    wings_wm:translation_change();
 		num_buttons ->
 		    wings_wm:translation_change();
-		no_progress_bar ->
-		    wings_pb:init();
 		language ->
-		    wings_lang:load_language(Val);
+		    case Val of
+		      V when V =:= "zh-cn"; V =:= "zh-tw"; V =:= "ko" ->
+		          delayed_set_value(Key, OldVal, Val),
+		          wings_u:message(?__(5,"The language change will take effect\nthe next time Wings 3D is started."));
+		      _ ->
+		          ets:delete_all_objects(system_font),
+		          ets:delete_all_objects(console_font),
+		          wings_lang:load_language(Val)
+		    end;
 		polygon_offset_f ->
 		    erase(polygon_offset);
 		polygon_offset_r ->
@@ -698,12 +707,6 @@ make_query({color,[_|_]=Str,Key}) ->
 make_query({color,[_|_]=Str,Key,Flags}) ->
     Def = get_value(Key),
     {Str,{color,Def,[{key,Key}|Flags]}};
-make_query({[_|_]=Str,text_display_lists=Key}) ->
-    %% Reverse sense to keep backwards comptibility of preferences.
-    {Str,not get_value(Key),[{key,Key}]};
-make_query({[_|_]=Str,text_display_lists=Key,Flags}) ->
-    %% Reverse sense to keep backwards comptibility of preferences.
-    {Str,not get_value(Key),[{key,Key}|Flags]};
 make_query({[_|_]=Str,Key}) ->
     case get_value(Key) of
 	Def when Def == true; Def == false ->
@@ -744,3 +747,12 @@ make_query({oframe,Frames,Key,Flags}) ->
 make_query(Tuple) when is_tuple(Tuple) ->
     list_to_tuple([make_query(El) || El <- tuple_to_list(Tuple)]);
 make_query(Other) -> Other.
+
+set_console() ->
+	case wings_wm:is_window(console) of
+	  true ->
+		{Pos,Size} = wings_wm:win_rect(console),
+		wings_console:window(),
+		wings_wm:move(console, Pos, Size);
+	  false -> ok
+	end.
