@@ -4,7 +4,7 @@
 %%     Render all objects and helpers (such as axes) in the scene.
 %%     Used for the Geometry and AutoUV windows.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -17,8 +17,6 @@
 	 render/1,polygonOffset/1,
 	 enable_lighting/1,disable_lighting/0]).
 
--import(erlang, [max/2]).
-
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
@@ -188,14 +186,14 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 	    polygonOffset(2),
 	    gl:shadeModel(?GL_SMOOTH),
 	    enable_lighting(SceneLights),
-	    case Open of
-		false ->
-		    wings_dl:call(Faces);
-		true ->
+	    case wings_pref:get_value(show_backfaces) of
+		true when Open ->
 		    gl:lightModeli(?GL_LIGHT_MODEL_TWO_SIDE, ?GL_TRUE),
 		    gl:disable(?GL_CULL_FACE),
 		    wings_dl:call(Faces),
-		    gl:enable(?GL_CULL_FACE)
+		    gl:enable(?GL_CULL_FACE);
+		_ ->
+		    wings_dl:call(Faces)
 	    end,
 	    disable_lighting(),
 	    gl:shadeModel(?GL_FLAT);
@@ -293,9 +291,9 @@ render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth0,transparent=Trans0,
 	    gl:depthMask(?GL_TRUE)
     end,
 
-    case Open of
-	false -> ok;
-	true -> gl:disable(?GL_CULL_FACE)
+    case wings_pref:get_value(show_backfaces) of
+	true when Open -> gl:disable(?GL_CULL_FACE);
+	_ -> ok
     end,
     case {Smooth,RenderTrans} of
 	{none,false} ->   wings_dl:call(Work);
@@ -331,6 +329,7 @@ wire(#we{id=Id}) ->
     W = wings_wm:get_prop(wireframed_objects),
     gb_sets:is_member(Id, W).
 
+draw_sel(#dlo{sel=none}) -> ok;
 draw_sel(#dlo{sel=SelDlist,src_sel={edge,_}}) ->
     gl:lineWidth(wings_pref:get_value(selected_edge_width)),
     sel_color(),
@@ -339,19 +338,27 @@ draw_sel(#dlo{sel=SelDlist,src_sel={vertex,_}}) ->
     gl:pointSize(wings_pref:get_value(selected_vertex_size)),
     sel_color(),
     wings_dl:call(SelDlist);
-draw_sel(#dlo{orig_sel=OrigSel,sel=SelDlist}) ->
+draw_sel(#dlo{open=Open,sel=SelDlist}) ->
     sel_color(),
+    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     gl:enable(?GL_POLYGON_OFFSET_FILL),
     gl:polygonOffset(1, 1),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    case OrigSel =/= none orelse wings_pref:get_value(selection_style) =:= solid of
-	true -> 				%Solid selection style.
-	    wings_dl:call(SelDlist);
-	false ->				%Stippled selection style.
-	    gl:enable(?GL_POLYGON_STIPPLE),
-	    wings_dl:call(SelDlist),
-	    gl:disable(?GL_POLYGON_STIPPLE)
-    end.
+    %Stippled selection style.
+    gl:enable(?GL_POLYGON_STIPPLE),
+    draw_face_sel(Open, SelDlist),
+    gl:disable(?GL_POLYGON_STIPPLE).
+
+draw_face_sel(true, SelDlist) ->
+    case wings_pref:get_value(show_backfaces) of
+        true ->
+            gl:disable(?GL_CULL_FACE),
+            wings_dl:call(SelDlist),
+            gl:enable(?GL_CULL_FACE);
+        _ ->
+            wings_dl:call(SelDlist)
+    end;
+draw_face_sel(false, SelDlist) ->
+    wings_dl:call(SelDlist).
 
 sel_color() ->
     gl:color3fv(wings_pref:get_value(selected_color)).
@@ -388,13 +395,13 @@ draw_orig_sel_1(edge, DlistSel) ->
     wings_dl:call(DlistSel),
     gl:disable(?GL_BLEND);
 draw_orig_sel_1(_, DlistSel) ->
-    sel_color(),
-    gl:enable(?GL_POLYGON_STIPPLE),
-    gl:enable(?GL_POLYGON_OFFSET_FILL),
+    gl:enable(?GL_BLEND),
+    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
+    {R0,G0,B0} = wings_pref:get_value(selected_color),
+    gl:color4f(R0, G0, B0, 0.5),
     gl:polygonOffset(1, 1),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    wings_dl:call(DlistSel),
-    gl:disable(?GL_POLYGON_STIPPLE).
+    wings_dl:call(DlistSel).
 
 draw_hard_edges(#dlo{hard=none}, _) -> ok;
 draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
@@ -476,8 +483,7 @@ dummy_axis_letter(_, _, {_,_,W,H}) ->
     gl:matrixMode(?GL_MODELVIEW).
 
 axis_letters() ->
-    case wings_pref:get_value(show_axis_letters) andalso
-	wings_wm:get_prop(show_axes) of
+    case wings_wm:get_prop(show_axes) of
 	false ->
 	    case wings_pref:get_value(dummy_axis_letter) of
 		false -> ok;
