@@ -3,7 +3,7 @@
 %%
 %%     This module implements selection utilities.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -14,6 +14,7 @@
 -module(wings_sel).
 
 -export([clear/1,reset/1,set/2,set/3,
+	 conditional_reset/1,
 	 map/2,fold/3,mapfold/3,
 	 make/3,valid_sel/1,valid_sel/3,
 	 center/1,bbox_center/1,bounding_box/1,bounding_boxes/1,
@@ -29,15 +30,23 @@ clear(St) ->
     St#st{sel=[],sh=false}.
 
 reset(#st{selmode=Mode}=St) ->
-    case wings_pref:get_value(smart_highlighting) of
-	false -> St#st{sel=[],sh=false};
-	true when Mode =:= body -> St#st{selmode=face,sel=[],sh=true};
-	true -> St#st{sel=[],sh=true}
+    case Mode of
+	body -> St#st{selmode=face,sel=[],sh=true};
+	_ -> St#st{sel=[],sh=true}
     end.
 
+conditional_reset(#st{sel=[]}=St) ->
+    reset(St);
+conditional_reset(St) ->
+    St#st{sel=[],sh=false}.
+
+set([], St) ->
+    clear(St);
 set(Sel, St) ->
     St#st{sel=sort(Sel),sh=false}.
 
+set(Mode, [], St) ->
+    clear(St#st{selmode=Mode});
 set(Mode, Sel, St) ->
     St#st{selmode=Mode,sel=sort(Sel),sh=false}.
 
