@@ -3,7 +3,7 @@
 %%
 %%     This module implements the commands in the selection menu.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -16,13 +16,18 @@
 -export([menu/1,command/2]).
 
 %% Utilities.
--export([select_all/1]).
+-export([init/0,select_all/1]).
 
 -include("wings.hrl").
 -import(lists, [map/2,foldl/3,reverse/1,keymember/3,keyfind/3,usort/1]).
--import(erlang, [max/2]).
+
+init() ->
+    wings_pref:set_default(saved_selections_cycle_by_mode,false).
 
 menu(St) ->
+    Help = ?__(99," (from selection or all visible objects (if no selection))"),
+    RHelp = random_help(),
+    Objects = ?__(106," Objects"),
     [{?__(1,"Deselect"),deselect,?__(2,"Clear the selection")},
      separator,
      {?__(3,"More"),more,more_help(St)},
@@ -35,11 +40,16 @@ menu(St) ->
       {edge_loop,
        [{?__(7,"Edge Loop"),
 	 edge_loop,?__(8,"Expand edge selection to loop; ")++
-	 ?__(9,"convert face selection to selected border edges")},
+	 ?__(9,"convert face selection to selected border edges")++
+	 ?__(98,"; convert consecutive vertices to edges")},
 	{?__(10,"Edge Loop to Region"),edge_loop_to_region,
 	 ?__(11,"Select all faces on one side of an edge loop")},
 	{?__(12,"Edge Ring"),
 	 edge_ring,?__(13,"Expand edge selection to ring")},
+	{?__(100,"Every Nth Ring"),{nth_edge_ring,
+	    [{?__(101,"Second"),2},
+	     {?__(102,"Third"),3},
+	     {?__(103,"Nth..."),nth}]}},
 	separator,
 	{?__(14,"Previous Edge Loop"),
 	 prev_edge_loop,?__(15,"Select the previous edge loop")},
@@ -65,43 +75,47 @@ menu(St) ->
 		 {?__(30,"Objects"),body}]}},
      {?__(31,"By"),
       {by,[{?__(32,"Hard Edges"),
-	    hard_edges,?__(33,"Select all hard edges")},
+	    hard_edges,?__(33,"Select all hard edges")++Help},
 	   {?__(34,"Isolated Vertices"),
-	    isolated_vertices,?__(35,"Select all isolated vertices")},
+	    isolated_vertices,?__(35,"Select all isolated vertices")++Help},
 	   {?__(85,"Non-planar Faces"),
-	    nonplanar_faces,?__(86,"Select all non-planar faces"),[option]},
+	    nonplanar_faces,?__(86,"Select all non-planar faces")++Help,[option]},
 	   {?__(36,"Vertices With"),
 	    {vertices_with,
-	     [{?__(37,"2 Edges"),2},
-	      {?__(38,"3 Edges"),3},
-	      {?__(39,"4 Edges"),4},
-	      {?__(40,"5 Edges"),5},
-	      {?__(401,"6 or More"),6}]}},
+	     [{?__(37,"2 Edges"),2,Help},
+	      {?__(38,"3 Edges"),3,Help},
+	      {?__(39,"4 Edges"),4,Help},
+	      {?__(40,"5 Edges"),5,Help},
+	      {?__(401,"6 or More"),6,Help},
+	      {?__(402,"Specify..."),specify,Help}]}},
 	   {?__(41,"Faces With"),
 	    {faces_with,
-	     [{?__(42,"2 Edges"),2},
-	      {?__(43,"3 Edges"),3},
-	      {?__(44,"4 Edges"),4},
-	      {?__(45,"5 or More"),5}]}},
+	     [{?__(42,"2 Edges"),2,Help},
+	      {?__(43,"3 Edges"),3,Help},
+	      {?__(44,"4 Edges"),4,Help},
+	      {?__(45,"5 or More"),5,Help},
+	      {?__(402,"Specify..."),specify,Help}]}},
 	   {?__(nq0,"Non Quadrangle Faces"),
 	    {non_quad,
-	     [{?__(nq1,"All Non Quadrangle Faces"),all},
-	      {?__(nq2,"Odd Non Quadrangle Faces"),odd},
-	      {?__(nq3,"Even Non Quadrangle Faces"),even}]}},
+	     [{?__(nq1,"All Non Quadrangle Faces"),all,Help},
+	      {?__(nq2,"Odd Non Quadrangle Faces"),odd,Help},
+	      {?__(nq3,"Even Non Quadrangle Faces"),even,Help}]}},
 	   {?__(46,"Random"),
-	    {random,[{"10%",10},
-		     {"20%",20},
-		     {"30%",30},
-		     {"40%",40},
-		     {"50%",50},
-		     {"60%",60},
-		     {"70%",70},
-		     {"80%",80},
-		     {"90%",90}]}},
+	    {random,[{"10%",10, RHelp},
+		     {"20%",20, RHelp},
+		     {"30%",30, RHelp},
+		     {"40%",40, RHelp},
+		     {"50%",50, RHelp},
+		     {"60%",60, RHelp},
+		     {"70%",70, RHelp},
+		     {"80%",80, RHelp},
+		     {"90%",90, RHelp},
+		     {"__%",pick, RHelp}
+		     ]}},
 	   {?__(56,"Short Edges"),
-	    short_edges,?__(57,"Select (too) short edges"),[option]},
+	    short_edges,?__(57,"Select (too) short edges")++Help,[option]},
 	   {?__(87,"Sharp Edges"),
-	    sharp_edges,?__(88,"Select sharp edges"),[option]},
+	    sharp_edges,?__(88,"Select sharp edges")++Help,[option]},
 	   {?__(95,"Vertex Path"),
 	    {vertex_path,
 	     [{?__(89,"Fewest Edges Path"),
@@ -111,9 +125,9 @@ menu(St) ->
 	      {?__(93,"Shortest Path (A-Star)"),
 	       astar_shortest_path,?__(94,"Select the shortest path between two vertices (A-Star)")}]}},
 	   {?__(58,"Material Edges"),material_edges,
-	    ?__(59,"Select all edges between different materials")},
+	    ?__(59,"Select all edges between different materials")++Help},
 	   {?__(60,"UV-Mapped Faces"),uv_mapped_faces,
-	    ?__(61,"Select all edges that have UV coordinates")},
+	    ?__(61,"Select all edges that have UV coordinates")++Help},
 	   {?__(62,"Id..."),id,?__(63,"Select by numeric id")}]}},
      {?__(64,"Lights"),lights,?__(65,"Select all lights")},
      separator,
@@ -121,17 +135,17 @@ menu(St) ->
      separator,
      {?__(67,"Inverse"),inverse,?__(68,"Invert the selection")},
      separator,
-     {?__(69,"Hide Selected"),
+     {?__(69,"Hide Selected")++Objects,
       hide_selected,
       ?__(70,"Hide all (partly or wholly) selected objects")},
-     {?__(71,"Hide Unselected"),
+     {?__(71,"Hide Unselected")++Objects,
       hide_unselected,?__(72,"Hide objects that have no selection")},
-     {?__(73,"Lock Unselected"),
+     {?__(73,"Lock Unselected")++Objects,
       lock_unselected,?__(74,"Lock objects that have no selection")},
      separator,
-     {?__(75,"Show All"),
+     {?__(104,"Unhide All Objects"),
       show_all,?__(76,"Show all objects that have been hidden")},
-     {?__(77,"Unlock All"),
+     {?__(105,"Unlock All Objects"),
       unlock_all,?__(78,"Unlock all locked objects")},
      separator,
      {?__(79,"Store Selection"),store_selection,
@@ -141,6 +155,9 @@ menu(St) ->
      separator,
      {?__(83,"New Group..."),new_group,?__(84,"Create a new selection group")}|groups_menu(St)].
 
+random_help() ->
+    ?__(1,"Select random elements from current selection, or all visible objects (no selection)").
+
 sel_all_str(#st{selmode=vertex}) -> ?__(1,"All Vertices");
 sel_all_str(#st{selmode=edge}) -> ?__(2,"All Edges");
 sel_all_str(#st{selmode=face}) -> ?__(3,"All Faces");
@@ -166,32 +183,45 @@ similar_material_faces(_) ->
 
 groups_menu(#st{ssels=Ssels}=St) ->
     case gb_trees:is_empty(Ssels) of
-	true -> [];
-	false ->
-	    [{?__(1,"Delete Group"),
-	      {delete_group,
-	       groups_and_help(?__(2,"Delete group \""), "\"", St)}},
-	     separator,
-	     {?__(4,"Add to Group"),
-	      {add_to_group,
-	       groups_and_help(?__(5,"Add current selection to group \""),"\"", St)}},
-	     {?__(7,"Subtract from Group"),
-	      {subtract_from_group,
-	       groups_and_help(?__(8,"Subtract current selection from group \""),"\"", St)}},
-	     separator,
-	     {?__(10,"Select Group"),
-	      {select_group,
-	       groups_and_help(?__(11,"Select group \""), "\"", St)}},
-	     separator,
-	     {?__(13,"Union Group"),
-	      {union_group,
-	       groups_and_help(?__(14,"Union group \""),?__(15,"\" with current selection"), St)}},
-	     {?__(16,"Subtract Group"),
-	      {subtract_group,
-	       groups_and_help(?__(17,"Subtract group \""),?__(18,"\" from current selection"), St)}},
-	     {?__(19,"Intersect Group"),
-	      {intersect_group,
-	       groups_and_help(?__(20,"Intersect group \""),?__(21,"\" with current selection"), St)}}]
+        true -> [];
+        false ->
+          [{?__(22,"Selection Groups"),
+            {ssels,
+             [{?__(1,"Delete Group"),
+               {delete_group,
+                groups_and_help(?__(2,"Delete group \""), "\"", St)}},
+               separator,
+               {?__(4,"Add to Group"),
+                {add_to_group,
+                 groups_and_help(?__(5,"Add current selection to group \""),
+                   "\"", St)}},
+               {?__(7,"Subtract from Group"),
+                {subtract_from_group,
+                 groups_and_help(?__(8,"Subtract current selection from group \""),
+                   "\"", St)}},
+               separator,
+               {?__(10,"Select Group"),
+                {select_group,
+                 groups_and_help(?__(11,"Select group \""), "\"", St)++
+                 [separator,
+                 {?__(24,"Next Group"),next},
+                 {?__(25,"Previous Group"),prev},
+                 {?__(26,"Cycle In Selection Mode"),saved_selections_cycle_by_mode,
+                  ?__(27,"Cycle Prev/Next only within active selection mode"),
+                  wings_menu_util:crossmark(saved_selections_cycle_by_mode)}]}},
+               separator,
+               {?__(13,"Union Group"),
+                {union_group,
+                 groups_and_help(?__(14,"Union group \""),
+                   ?__(15,"\" with current selection"), St)}},
+               {?__(16,"Subtract Group"),
+                {subtract_group,
+                 groups_and_help(?__(17,"Subtract group \""),
+                   ?__(18,"\" from current selection"), St)}},
+               {?__(19,"Intersect Group"),
+                {intersect_group,
+                 groups_and_help(?__(20,"Intersect group \""),
+                   ?__(21,"\" with current selection"), St)}}]}}]
     end.
 
 groups_and_help(Help0, Help1, #st{ssels=Ssels}) ->
@@ -233,6 +263,8 @@ similar_help(#st{selmode=face}) ->
 similar_help(#st{selmode=body}) ->
     ?__(4,"Select objects with the same number of edges, faces, and vertices").
 
+command({edge_loop,edge_loop}, #st{selmode=vertex}=St) ->
+    {save_state,vs_to_edge_loop(St)};
 command({edge_loop,edge_loop}, #st{selmode=face}=St) ->
     {save_state,face_region_to_edge_loop(St)};
 command({edge_loop,edge_loop}, St) ->
@@ -241,6 +273,10 @@ command({edge_loop,edge_link_decr}, St) ->
     {save_state,wings_edge_loop:select_link_decr(St)};
 command({edge_loop,edge_link_incr}, St) ->
     {save_state,wings_edge_loop:select_link_incr(St)};
+command({edge_loop,{nth_edge_ring,nth}}, St) ->
+    select_nth_ring(St);
+command({edge_loop,{nth_edge_ring,N}}, St) ->
+    {save_state,wings_edge:select_nth_ring(N,St)};
 command({edge_loop,edge_ring}, St) ->
     {save_state,wings_edge:select_edge_ring(St)};
 command({edge_loop,edge_ring_incr}, St) ->
@@ -256,11 +292,11 @@ command({edge_loop,edge_loop_to_region}, St) ->
 command({edge_loop,complete_loops}, St) ->
     {save_state,complete_loops(St)};
 command(deselect, St) ->
-    {save_state,wings_sel:reset(St)};
+    {save_state,deselect(St)};
 command(more, St) ->
-    wings_sel_conv:more(St);
+    {save_state,wings_sel_conv:more(St)};
 command(less, St) ->
-    wings_sel_conv:less(St);
+    {save_state,wings_sel_conv:less(St)};
 command(all, St) ->
     {save_state,select_all(St)};
 command(lights, St) ->
@@ -276,24 +312,35 @@ command({similar_area,Ask}, St) ->
 	
 command({similar_material,Ask}, St) ->
     similar_material(Ask, St);
+
+command({ssels,{select_group,saved_selections_cycle_by_mode}}, St) ->
+    Pref = wings_pref:get_value(saved_selections_cycle_by_mode),
+    wings_pref:set_value(saved_selections_cycle_by_mode, not Pref),
+    {save_state,St};
+command({ssels,{select_group,Id}}, St) when Id =:= next; Id =:= prev ->
+    {save_state,cycle_group(Id, St)};
 	
-command({select_group,Id}, St) ->
+command({ssels,{select_group,Id}}, St) ->
     {save_state,select_group(Id, St)};
-command({union_group, Id}, St) ->
+command({ssels,{union_group,Id}}, St) ->
     {save_state,union_group(Id, St)};
-command({subtract_group, Id}, St) ->
+command({ssels,{subtract_group,Id}}, St) ->
     {save_state,subtract_group(Id, St)};
-command({intersect_group, Id}, St) ->
+command({ssels,{intersect_group,Id}}, St) ->
     {save_state,intersect_group(Id, St)};
-command({add_to_group, Id}, St) ->
+command({ssels,{add_to_group,Id}}, St) ->
     {save_state,add_to_group(Id, St)};
-command({subtract_from_group, Id}, St) ->
+command({ssels,{subtract_from_group,Id}}, St) ->
     {save_state,subtract_from_group(Id, St)};
 command({new_group_name, Name}, St) ->
     {save_state,new_group_name(Name, St)};
 command(new_group, St) ->
     new_group(St);
-command({delete_group,Id}, #st{ssels=Ssels}=St) ->
+command({ssels,{delete_group,invalid}}, St) ->
+    {save_state,delete_invalid_groups(St)};
+command({ssels,{delete_group,all}}, St) ->
+    {save_state,St#st{ssels=gb_trees:empty()}};
+command({ssels,{delete_group,Id}}, #st{ssels=Ssels}=St) ->
     {save_state,St#st{ssels=gb_trees:delete(Id, Ssels)}};
 command(inverse, St) ->
     {save_state,inverse(St)};
@@ -325,62 +372,25 @@ command(Type, St) ->
     set_select_mode(Type, St).
 
 by_command(hard_edges, St) ->
-    Sel = fun(Edge, #we{he=Htab}) ->
-		  gb_sets:is_member(Edge, Htab)
-	  end,
-    {save_state,wings_sel:make(Sel, edge, St)};
+    hard_edges(St);
 by_command(isolated_vertices, St) ->
     {save_state,select_isolated(St)};
 by_command({nonplanar_faces,Ask}, St) ->
     nonplanar_faces(Ask, St);
-by_command({vertices_with,6}, St) ->
-    Sel = fun(V, We) ->
-		  Cnt = wings_vertex:fold(
-			  fun(_, _, _, Cnt) ->
-				  Cnt+1
-			  end, 0, V, We),
-		  Cnt >= 6
-	  end,
-    {save_state,wings_sel:make(Sel, vertex, St)};
 by_command({vertices_with,N}, St) ->
-    Sel = fun(V, We) ->
-		  Cnt = wings_vertex:fold(
-			  fun(_, _, _, Cnt) ->
-				  Cnt+1
-			  end, 0, V, We),
-		  Cnt =:= N
-	  end,
-    {save_state,wings_sel:make(Sel, vertex, St)};
+    vertices_with(N, St);
 by_command({non_quad,all}, St) ->
-    Sel = fun(Face, We) ->
-		  wings_face:vertices(Face, We) =/= 4
-	    end,
-    {save_state,wings_sel:make(Sel, face, St)};
+    {save_state,faces_with({non_quad,all}, St)};
 by_command({non_quad,odd}, St) ->
-    Sel = fun(Face, We) ->
-		  wings_face:vertices(Face, We) rem 2 =/= 0
-	  end,
-    {save_state,wings_sel:make(Sel, face, St)};
+    {save_state,faces_with({non_quad,odd}, St)};
 by_command({non_quad,even}, St) ->
-    Sel = fun(Face, We) ->
-		  N = wings_face:vertices(Face, We),
-		  N =/= 4 andalso N rem 2 =:= 0
-	  end,
-    {save_state,wings_sel:make(Sel, face, St)};
-by_command({faces_with,5}, St) ->
-    Sel = fun(Face, We) ->
-		  wings_face:vertices(Face, We) >= 5
-	    end,
-    {save_state,wings_sel:make(Sel, face, St)};
+    {save_state,faces_with({non_quad,even}, St)};
 by_command({faces_with,N}, St) ->
-    Sel = fun(Face, We) ->
-		  N =:= wings_face:vertices(Face, We)
-	  end,
-    {save_state,wings_sel:make(Sel, face, St)};
+    faces_with({faces_with,N}, St);
 by_command(material_edges, St) ->
     material_edges(St);
-by_command({random,Percent}, St) ->
-    {save_state,random(Percent, St)};
+by_command({random, Percent}, St) ->
+    random(Percent, St);
 by_command({short_edges,Ask}, St) ->
     short_edges(Ask, St);
 by_command({sharp_edges,Ask}, St) ->
@@ -412,6 +422,32 @@ subtract_mirror_edges(Es, #we{mirror=none}) -> Es;
 subtract_mirror_edges(Es, #we{mirror=Face}=We) ->
     Es -- wings_face:to_edges([Face], We).
 
+vs_to_edge_loop(St) ->
+    Sel = wings_sel:fold(
+      fun(Vs, #we{id=Id}=We, Acc) ->
+        Es0 = vs_to_edges(Vs, We, []),
+        Es1 = subtract_mirror_edges(Es0, We),
+        Es = gb_sets:from_list(Es1),
+        [{Id,Es}|Acc]
+      end, [], St),
+    wings_sel:set(edge, Sel, St).
+
+vs_to_edges(Vs0, We, Es0) ->
+    case gb_sets:is_empty(Vs0) of
+      true -> lists:usort(Es0);
+      false ->
+        {Va,Vs} = gb_sets:take_smallest(Vs0),
+        Es = wings_vertex:fold(
+          fun(Edge, _, EdgeRec, Es1) ->
+            Vb = wings_vertex:other(Va, EdgeRec),
+            case gb_sets:is_element(Vb, Vs) of
+              true -> [Edge|Es1];
+              _ -> Es1
+            end
+          end, Es0, Va, We),
+        vs_to_edges(Vs, We, Es)
+    end.
+
 %%%
 %%% Selection commands.
 %%%
@@ -454,6 +490,16 @@ inverse(#st{selmode=Mode}=St) ->
 	    end, [], St),
     St#st{sel=reverse(Sel)}.
 
+%%%
+%%% Deselect
+%%%
+
+deselect(St) ->
+    case wings_pref:get_value(conditional_deselect) of
+      true -> wings_sel:conditional_reset(St);
+      false -> wings_sel:reset(St)
+    end.
+
 %%%
 %%% Hide Selected
 %%% Hide Unselected
@@ -494,7 +540,7 @@ update_unsel(Perm, #st{shapes=Shs0,sel=Sel}=St) ->
 union_group(Key, #st{sel=Sel0}=St) ->
     Ssel = coerce_ssel(Key, St),
     Sel = union(Sel0, Ssel),
-    St#st{sel=Sel}.
+    wings_sel:valid_sel(St#st{sel=Sel}).
 
 union(Sa, Sb) ->
     combine_sel(fun(Ss) -> gb_sets:union(Ss) end, Sa, Sb).
@@ -564,7 +610,25 @@ select_group({Mode,_}=Key, #st{ssels=Ssels}=St) ->
     ValidSel = wings_sel:valid_sel(Ssel, Mode, St),
     St#st{selmode=Mode,sel=ValidSel}.
 
-add_to_group({Mode,_}=Key, #st{ssels=Ssels}=St) ->
+%%%% Delete Groups that return an empty selection. Invalid ssels can result from
+%%%% creating or deleting geomerty.
+delete_invalid_groups(#st{ssels=Ssels}=St0) ->
+    case gb_trees:is_empty(Ssels) of
+      true ->
+        St0;
+      false ->
+        Keys = gb_trees:keys(Ssels),
+        lists:foldl(fun(Key,#st{ssels=Ss,selmode=Mode}=St) ->
+                Ssel = gb_trees:get(Key,Ss),
+                ValidSel = wings_sel:valid_sel(Ssel, Mode, St),
+                case ValidSel of
+                  [] -> St#st{ssels=gb_trees:delete(Key,Ss)};
+                  _ -> St
+                end
+        end,St0,Keys)
+    end.
+
+add_to_group({Mode,_}=Key, #st{ssels=Ssels}=St) -> 
     Ssel0 = gb_trees:get(Key, Ssels),
     Ssel1 = wings_sel:valid_sel(Ssel0, Mode, St),
     #st{sel=Sel} = possibly_convert(Mode, St),
@@ -601,7 +665,7 @@ new_group_name(Name, #st{ssels=Ssels0,selmode=Mode,sel=Sel}=St) ->
 	    Exists = ?__(exists,"already exists."),
 	    Msg0 = [GroupMode," \"",Name,"\" ",Exists],
 	    Msg = lists:flatten(Msg0),
-	    wings_u:error(Msg)
+	    wings_u:error_msg(Msg)
     end,
     Ssels = gb_trees:insert(Key, Sel, Ssels0),
     St#st{ssels=Ssels}.
@@ -615,6 +679,54 @@ group_mode_string(face) ->
 group_mode_string(body) ->
     ?__(body, "Body selection group").
 
+%%%% Cycle Through Save Selections
+cycle_group(Dir, #st{selmode=SelMode,ssels=Ssels,sh=Sh}=St) ->
+    case gb_trees:is_empty(Ssels) of
+      true -> St;
+      false ->
+        Keys0 = gb_trees:keys(Ssels),
+        Keys1 = case wings_pref:get_value(saved_selections_cycle_by_mode) of
+          true when Sh -> Keys0;
+          true -> [Key || {Mode,_}=Key <- Keys0, Mode =:= SelMode];
+          false -> Keys0
+        end,
+        Keys = case Dir of
+          next -> Keys1;
+          prev -> lists:reverse(Keys1)
+        end,
+        cycle_ss_keys(Keys,St)
+    end.
+
+cycle_ss_keys([],St) -> St;
+cycle_ss_keys(Keys,St) ->
+    case search_ssel_keys(Keys,St,[]) of
+      {none,[]} -> [Key|_] = Keys;
+      {none,Acc} ->
+        Key = lists:last(Acc);
+      [] ->
+        [Key|_] = Keys;
+      Other ->
+        [Key|_] = Other
+    end,
+    case select_group(Key,St) of
+      #st{sel=[]} -> cycle_ss_keys(lists:delete(Key,Keys),St);
+      NewSt -> NewSt
+    end.
+
+search_ssel_keys([{Mode,_}=PKey|Keys],#st{selmode=Mode,sel=Sel,ssels=Ssels}=St,Acc) ->
+    PSel0 = gb_trees:get(PKey,Ssels),
+    PSel = wings_sel:valid_sel(PSel0, Mode, St),
+    case PSel =:= Sel of
+      true ->
+        Keys;
+      false ->
+        search_ssel_keys(Keys,St,[PKey|Acc])
+    end;
+search_ssel_keys([_|Keys],St,Acc) ->
+    search_ssel_keys(Keys,St,Acc);
+search_ssel_keys([],_St,Acc) ->
+    {none,Acc}.
+
 %%%
 %%% Select Similar.
 %%%
@@ -763,9 +875,28 @@ compare(A, B) ->
 %% Select Random.
 %%
 
-random(Percent, #st{selmode=Mode}=St) ->
+random(pick, _) ->
+    Qs = [{hframe,[{slider,{text,wings_pref:get_value(random_select, 25.0),
+            [{range,{0.0,100.0}}]}}]}],
+    wings_ask:dialog(true, ?__(1,"Select Random"),
+    [{vframe,Qs}],
+    fun([Res]) ->
+      wings_pref:set_value(random_select, Res),
+      {select,{by,{random,Res}}} end);
+random(Percent, #st{selmode=Mode, sel=[]}=St) ->
     P = Percent / 100,
-    wings_sel:make(fun(_, _) -> random:uniform() < P end, Mode, St).
+    {save_state, wings_sel:make(fun(_, _) -> random:uniform() < P end, Mode, St)};
+random(Percent, St) ->
+    P = Percent / 100,
+    NewSel = wings_sel:fold(fun(Sel0, #we{id=Id}, Acc) ->
+            Sel1 = gb_sets:to_list(Sel0),
+            Sel2 = [Elem || Elem <- Sel1, random:uniform() < P ],
+            case Sel2 of
+              [] -> Acc;
+              _ -> [{Id,gb_sets:from_list(Sel2)}|Acc]
+            end
+        end,[],St),
+    {save_state, wings_sel:set(NewSel, St)}.
 
 %%
 %% Select short edges.
@@ -777,11 +908,22 @@ short_edges(Ask, _St) when is_atom(Ask) ->
     wings_ask:dialog(Ask, ?__(2,"Select Short Edges"),
 		     [{hframe,Qs}],
 		     fun(Res) -> {select,{by,{short_edges,Res}}} end);
-short_edges([Tolerance], St0) ->
+short_edges([Tolerance], #st{sel=[]}=St0) ->
     St = wings_sel:make(fun(Edge, We) ->
 				short_edge(Tolerance, Edge, We)
 			end, edge, St0),
-    {save_state,St#st{selmode=edge}}.
+    {save_state,St#st{selmode=edge}};
+short_edges([Tolerance], #st{selmode=Mode}=St0) ->
+    St = if Mode =:= edge -> St0; true -> wings_sel_conv:mode(edge, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, Acc) ->
+				Sel1 = gb_sets:to_list(Sel0),
+				ShortEdges = [Edge || Edge <- Sel1, short_edge(Tolerance, Edge, We)],
+				case ShortEdges of
+				  [] -> Acc;
+				  _ -> [{Id,gb_sets:from_list(ShortEdges)}|Acc]
+				end
+			end, [], St),
+    {save_state,wings_sel:set(edge,Sel,St0)}.
 
 short_edge(Tolerance, Edge, #we{es=Etab,vp=Vtab}) ->
     #edge{vs=Va,ve=Vb} = array:get(Edge, Etab),
@@ -793,8 +935,19 @@ short_edge(Tolerance, Edge, #we{es=Etab,vp=Vtab}) ->
 %% Select all edges between materials.
 %%
 
-material_edges(St) ->
-    wings_sel:make(fun material_edges_fun/2, edge, St).
+material_edges(#st{sel=[]}=St) ->
+    wings_sel:make(fun material_edges_fun/2, edge, St);
+material_edges(#st{selmode=Mode}=St0) ->
+    St = if Mode =:= edge -> St0; true -> wings_sel_conv:mode(edge, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We,Acc) ->
+		  Sel1 = gb_sets:to_list(Sel0),
+		  MatEdges = [Edge || Edge <- Sel1, material_edges_fun(Edge, We)],
+		  case MatEdges of
+		    [] -> Acc;
+		    _ -> [{Id, gb_sets:from_list(MatEdges)}|Acc]
+		  end
+	  end,[],St),
+    wings_sel:set(edge, Sel, St0).
 
 material_edges_fun(E, #we{es=Etab}=We) ->
     #edge{lf=Lf,rf=Rf} = array:get(E, Etab),
@@ -804,14 +957,27 @@ material_edges_fun(E, #we{es=Etab}=We) ->
 %% Select all faces that have (proper) UV coordinates.
 %%
 
-uv_mapped_faces(#st{shapes=Shs}=St) ->
+uv_mapped_faces(#st{shapes=Shs,sel=[]}=St) ->
     Sel = foldl(fun(#we{id=Id}=We, A) ->
 			case wings_we:uv_mapped_faces(We) of
 			    [] -> A;
 			    Fs -> [{Id,gb_sets:from_ordset(Fs)}|A]
 			end
 		end, [], gb_trees:values(Shs)),
-    wings_sel:set(face, Sel, St).
+    wings_sel:set(face, Sel, St);
+uv_mapped_faces(#st{selmode=Mode}=St0) ->
+    St = if Mode =:= face -> St0; true -> wings_sel_conv:mode(face, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, Acc) ->
+				Sel1 = gb_sets:to_list(Sel0),
+				UVF = wings_we:uv_mapped_faces(We),
+				Faces = [Face || Face <- Sel1, lists:member(Face, UVF)],
+				case Faces of
+				  [] -> Acc;
+				  _ -> [{Id,gb_sets:from_list(Faces)}|Acc]
+				end
+			end, [], St),
+	wings_sel:set(face,Sel,St0).
+
 
 %%
 %% Select by numerical item id.
@@ -836,7 +1002,7 @@ item_by_id(Prompt, #st{sel=[{Id,_}]}) ->
 	end);
 item_by_id(Prompt, #st{shapes=Shs}) ->
     case gb_trees:to_list(Shs) of
-	[] -> wings_u:error(?__(1,"Nothing to select."));
+	[] -> wings_u:error_msg(?__(1,"Nothing to select."));
 	[{Id,_}] ->
 	    ask([{Prompt,0}],
 		fun([Item]) ->
@@ -860,11 +1026,11 @@ valid_sel(Prompt, Sel, #st{shapes=Shs,selmode=Mode}=St) ->
 	    [Item] = gb_sets:to_list(Item0),
 	    case gb_trees:is_defined(Id, Shs) of
 		false ->
-		    wings_u:error(?__(1,"The Object Id ")++
+		    wings_u:error_msg(?__(1,"The Object Id ")++
 				  integer_to_list(Id)++
 				  ?__(2," is invalid."));
 		true ->
-		    wings_u:error(?__(3,"The ")++Prompt++" "++
+		    wings_u:error_msg(?__(3,"The ")++Prompt++" "++
 				  integer_to_list(Item)++
 				  ?__(4," is invalid."))
 	    end;
@@ -905,11 +1071,22 @@ select_lights_1([], _) -> [].
 %%% Select isolated vertices.
 %%%
 
-select_isolated(#st{shapes=Shs}=St) ->
+select_isolated(#st{shapes=Shs, sel=[]}=St) ->
     Sel = foldl(fun(#we{perm=Perm}=We, A) when ?IS_SELECTABLE(Perm) ->
 			select_isolated_1(We, A);
 		   (_, A) -> A
 		end, [], gb_trees:values(Shs)),
+    wings_sel:set(vertex, Sel, St);
+select_isolated(#st{selmode=Mode}=St0) ->
+    St = if Mode =:= vertex -> St0; true -> wings_sel_conv:mode(vertex, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, A) ->
+			Isolated0 = gb_sets:from_list(wings_vertex:isolated(We)),
+			Isolated = gb_sets:intersection(Sel0, Isolated0),
+			case gb_sets:is_empty(Isolated) of
+			  true -> A;
+			  false -> [{Id,Isolated}|A]
+			end
+		end, [], St),
     wings_sel:set(vertex, Sel, St).
 
 select_isolated_1(#we{id=Id}=We, A) ->
@@ -929,11 +1106,23 @@ nonplanar_faces(Ask, _St) when is_atom(Ask) ->
     wings_ask:dialog(Ask, ?__(2,"Select Non-planar Faces"),
 		     [{hframe,Qs}],
 		     fun(Res) -> {select,{by,{nonplanar_faces,Res}}} end);
-nonplanar_faces([Tolerance], St) ->
+nonplanar_faces([Tolerance], #st{sel=[]}=St) ->
     Sel = fun(Face, We) ->
 		  not wings_face:is_planar(Tolerance,Face,We)
 	  end,
-    {save_state,wings_sel:make(Sel, face, St)}.
+    {save_state,wings_sel:make(Sel, face, St)};
+nonplanar_faces([Tolerance], #st{selmode=Mode}=St0) ->
+    St = if Mode =:= face -> St0; true -> wings_sel_conv:mode(face, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, Acc) ->
+		  Sel1 = gb_sets:to_list(Sel0),
+		  NPFaces = [Face || Face <- Sel1, not wings_face:is_planar(Tolerance,Face,We)],
+		  case NPFaces of
+		    [] -> Acc;
+		    _ -> [{Id,gb_sets:from_list(NPFaces)}|Acc]
+		  end
+	  end,[],St),
+    {save_state,wings_sel:set(face, Sel, St0)}.
+
 
 %%%
 %%% Select similarly oriented faces.
@@ -960,7 +1149,7 @@ oriented_faces(Ask, _St) when is_atom(Ask) ->
 oriented_faces([Tolerance,Connected,Save], #st{selmode=face, sel=[]}) ->
     wings_pref:set_value(similar_normals_connected,Connected),
     wings_pref:set_value(similar_normals_angle,{Save,Tolerance}),
-    wings_u:error(?__(4,"At least one face must be selected"));
+    wings_u:error_msg(?__(4,"At least one face must be selected"));
 
 oriented_faces([Tolerance,false,Save], St) ->
     wings_pref:set_value(similar_normals_connected,false),
@@ -1029,21 +1218,23 @@ any_matching_normal(CosTolerance, Norm, [N|T]) ->
 %%% Select faces of the same material.
 %%%
 similar_material(_, #st{selmode=Mode}) when Mode =/= face ->
-    keep;					%Wrong mode (invoked through hotkey).
+    keep; %Wrong mode (invoked through hotkey).
 
 similar_material(Ask, _St) when is_atom(Ask) ->
-    Connected = wings_pref:get_value(similar_materials_connected,false),
+    Connected = wings_pref:get_value(similar_materials_connected, false),
+    Mode = wings_pref:get_value(similar_materials, material),
     Qs = [{?__(2,"Connected Faces Only"),Connected},
 	  {vradio,[{?__(5,"Materials"),material},
 		   {?__(6,"Vertex Color"),vertex_color}],
-	   material}],
+	   Mode}],
     wings_ask:dialog(Ask, ?__(4,"Select Faces with the same Material"),
     [{vframe,Qs}],
     fun(Res) -> {select,{similar_material,Res}} end);
 
-similar_material([Connected|_], #st{selmode=face,sel=[]}) ->
-    wings_pref:set_value(similar_materials_connected,Connected),
-    wings_u:error(?__(3,"At least one face must be selected"));
+similar_material([Connected,Mode], #st{selmode=face,sel=[]}) ->
+    wings_pref:set_value(similar_materials, Mode),
+    wings_pref:set_value(similar_materials_connected, Connected),
+    wings_u:error_msg(?__(3,"At least one face must be selected"));
 
 similar_material([false,Mode], St) ->
     Materials = wings_sel:fold(fun
@@ -1058,28 +1249,29 @@ similar_material([false,Mode], St) ->
     Sel = fun	
         (F, We) when Mode =:= vertex_color ->
           Col = average_colors(F, We),
-          any_matching_material(Col,Materials);
+          any_matching_material(Col, Materials);
         (Face, We) ->
-          Mat = wings_facemat:face(Face,We),
+          Mat = wings_facemat:face(Face, We),
           any_matching_material(Mat, Materials)
       end,
-    wings_pref:set_value(similar_materials_connected,false),
+    wings_pref:set_value(similar_materials, Mode),
+    wings_pref:set_value(similar_materials_connected, false),
     {save_state,wings_sel:make(Sel, face, St)};
 
 similar_material([true,Mode], St0) ->
     Selection = wings_sel:fold(fun
         (Faces, #we{id=Id}=We, A) when Mode =:= vertex_color ->
-            AllCols = gb_sets:fold(fun(F,Acc) ->
+            AllCols = gb_sets:fold(fun(F, Acc) ->
                 [average_colors(F, We)|Acc]
             end,A,Faces),
             Colours = lists:usort(AllCols),
-            [{Id,mat_search(Faces,Colours,We,Mode,Faces)}|A];
+            [{Id,mat_search(Faces, Colours, We, Mode, Faces)}|A];
         (Faces, #we{id=Id}=We, A) ->
             AllMats = [wings_facemat:face(Face, We) || Face <- gb_sets:to_list(Faces)],
             Materials = lists:usort(AllMats),
-            [{Id,mat_search(Faces,Materials,We,Mode,Faces)}|A]
+            [{Id,mat_search(Faces, Materials, We, Mode, Faces)}|A]
     end, [], St0),
-    wings_sel:set(face,Selection,St0).
+    wings_sel:set(face, Selection, St0).
 
 average_colors(Face, We) ->
     wings_color:average([C || C <- wings_va:face_attr(color, Face, We)]).
@@ -1126,25 +1318,53 @@ any_matching_material(Material, [Mat|T]) ->
 %%%
 
 sharp_edges(Ask, _St) when is_atom(Ask) ->
-    Qs = [{label,?__(1,"Max Angle")},
-	  {text,120.0,[{range,{0.0,180.0}}]}],
+    Qs = [{hframe,[{label,?__(1,"Max Angle")},
+      {slider,{text,120.0,[{range,{0.0,180.0}}]}}]},
+      {hradio,[{?__(3,"Peaks"),convex},
+               {?__(4,"Valleys"),concave},
+               {?__(5,"Both"),both}],both}],
     wings_ask:dialog(Ask,
-	?__(2,"Select Sharp Edges"), [{hframe,Qs}],
-	fun(Res) ->
-	    {select,{by,{sharp_edges,Res}}}
-	end);
-sharp_edges([Tolerance], St0) ->
+    ?__(2,"Select Sharp Edges"), [{vframe,Qs}],
+    fun(Res) ->
+        {select,{by,{sharp_edges,Res}}}
+    end);
+sharp_edges([Tolerance,Type], #st{sel=[]}=St0) ->
     CosTolerance = -math:cos(Tolerance * math:pi() / 180.0),
     St = wings_sel:make(fun(Edge, We) ->
-	     sharp_edge(CosTolerance, Edge, We)
-	 end, edge, St0),
-    {save_state,St}.
+         sharp_edge(CosTolerance, Type, Edge, We)
+     end, edge, St0),
+    {save_state,St};
+sharp_edges([Tolerance,Type], #st{selmode=Mode}=St0) ->
+    St = if Mode =:= edge -> St0; true -> wings_sel_conv:mode(edge, St0) end,
+    CosTolerance = -math:cos(Tolerance * math:pi() / 180.0),
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, Acc) ->
+          Sel1 = gb_sets:to_list(Sel0),
+          SharpEdges = [Edge || Edge <- Sel1, sharp_edge(CosTolerance, Type, Edge, We)],
+          case SharpEdges of
+            [] -> Acc;
+            _ -> [{Id, gb_sets:from_list(SharpEdges)}|Acc]
+          end
+      end,[],St),
+    {save_state,wings_sel:set(edge, Sel, St0)}.
 
-sharp_edge(CosTolerance, Edge, #we{es=Etab}=We) ->
-    #edge{lf=Lf,rf=Rf} = array:get(Edge, Etab),
+sharp_edge(CosTolerance, Type, Edge, #we{es=Etab,vp=Vtab}=We) ->
+    #edge{lf=Lf,rf=Rf,vs=Va,ve=Vb} = array:get(Edge, Etab),
     Lfn = wings_face:normal(Lf, Edge, We),
     Rfn = wings_face:normal(Rf, Edge, We),
-    e3d_vec:dot(Lfn,Rfn) < CosTolerance.
+    Dot = e3d_vec:dot(Lfn,Rfn),
+    case Dot < CosTolerance of
+      false -> false;
+      true when Type =:= both -> true;
+      true ->
+        EdgeNormal = e3d_vec:add(Lfn,Rfn),
+        EdgeCenter = e3d_vec:average(array:get(Va, Vtab), array:get(Vb, Vtab)),
+        FacesCenter = e3d_vec:average(wings_face:center(Lf,We),wings_face:center(Rf,We)),
+        Vec = e3d_vec:sub(EdgeCenter,FacesCenter),
+        case Type of
+           convex -> e3d_vec:dot(EdgeNormal,Vec) > 0.0;
+           concave -> e3d_vec:dot(EdgeNormal,Vec) < 0.0
+        end
+    end.
 
 %%%
 %%% Select shortest path
@@ -1154,12 +1374,12 @@ shortest_path(Method, St) ->
     #st{shapes=Shapes,selmode=Mode,sel=Sel} = St,
     case (Mode==vertex) andalso (length(Sel)==1) of
 	true -> ok;
-	false -> wings_u:error(?__(1,"Exactly two vertices must be\n selected on the same object."))
+	false -> wings_u:error_msg(?__(1,"Exactly two vertices must be\n selected on the same object."))
     end,
     [{Id,SelectedVs}] = Sel,
     case gb_sets:size(SelectedVs)==2 of
 	true -> ok;
-	false -> wings_u:error(?__(2,"Exactly two vertices must be selected."))
+	false -> wings_u:error_msg(?__(2,"Exactly two vertices must be selected."))
     end,
     We = gb_trees:get(Id, Shapes),
     [Pa,Pb] = [wings_vertex:pos(V, We) || V <- gb_sets:to_list(SelectedVs)],
@@ -1356,12 +1576,12 @@ similar_area([Tolerance], St) ->
     #st{shapes=Shapes,selmode=Mode,sel=Sel} = St,
     case (Mode==face) and (length(Sel)==1) of
 	true -> ok;
-	false -> wings_u:error(?__(3,"Exactly one face must be\n selected on a single object."))
+	false -> wings_u:error_msg(?__(3,"Exactly one face must be\n selected on a single object."))
     end,
     [{Id,SelectedFs}] = Sel,
     case gb_sets:size(SelectedFs)==1 of
 	true -> ok;
-	false -> wings_u:error(?__(4,"Exactly one face must be selected."))
+	false -> wings_u:error_msg(?__(4,"Exactly one face must be selected."))
     end,
     We = gb_trees:get(Id, Shapes),
     Face1 = hd(gb_sets:to_list(SelectedFs)),
@@ -1387,3 +1607,124 @@ complete_loops(#st{selmode=edge}=St) ->
 complete_loops(St0) ->
     St = wings_sel_conv:mode(edge,St0),
     wings_edge_loop:select_loop(St).
+
+hard_edges(#st{sel=[]}=St) ->
+    Sel = fun(Edge, #we{he=Htab}) ->
+		  gb_sets:is_member(Edge, Htab)
+	  end,
+    {save_state,wings_sel:make(Sel, edge, St)};
+hard_edges(#st{selmode=Mode}=St0) ->
+    St = if Mode =:= edge -> St0; true -> wings_sel_conv:mode(edge, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id,he=Htab},Acc) ->
+		  Sel1 = gb_sets:to_list(Sel0),
+		  HardEdges = [Edge || Edge <- Sel1, gb_sets:is_member(Edge, Htab)],
+		  case HardEdges of
+		    [] -> Acc;
+		    _ -> [{Id, gb_sets:from_list(HardEdges)}|Acc]
+		  end
+	  end,[],St),
+    {save_state,wings_sel:set(edge, Sel, St0)}.
+
+vertices_with(specify, _) ->
+    Qs = [{vframe,
+           [{hframe,[{label,?__(1,"Number of Edges")},
+             {text,2,[{range,{2,1000}}]}]},
+             {hradio,[{?__(3,"More"),more},
+               {?__(4,"Less"),less},
+               {?__(5,"Exactly"),exactly}],exactly}]}],
+    wings_ask:dialog(true,
+        ?__(2,"Select Vertices"), [{hframe,Qs}],
+        fun(Res) ->
+            {select,{by,{vertices_with,Res}}}
+        end);
+vertices_with(N, #st{sel=[]}=St) ->
+    Sel = fun(V, We) ->
+	  vertices_with(N, V, We)
+    end,
+    wings_sel:make(Sel, vertex, St);
+vertices_with(N, #st{selmode=Mode}=St0) ->
+    St = if Mode =:= vertex -> St0; true -> wings_sel_conv:mode(vertex, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We,Acc) ->
+		  Sel1 = gb_sets:to_list(Sel0),
+		  Vertices = [Vertex || Vertex <- Sel1, vertices_with(N, Vertex, We)],
+		  case Vertices of
+		    [] -> Acc;
+		    _ -> [{Id, gb_sets:from_list(Vertices)}|Acc]
+		  end
+	  end,[],St),
+    {save_state,wings_sel:set(vertex, Sel, St0)}.
+
+vertices_with(6, V, We) ->
+    Cnt = wings_vertex:fold(
+	  fun(_, _, _, Cnt) ->
+		  Cnt+1
+	  end, 0, V, We),
+    Cnt >= 6;
+vertices_with(N, V, We) when is_integer(N) ->
+    Cnt = wings_vertex:fold(
+	  fun(_, _, _, Cnt) ->
+		  Cnt+1
+	  end, 0, V, We),
+    Cnt =:= N;
+vertices_with([N,Mode], V, We) ->
+    Cnt = wings_vertex:fold(
+	  fun(_, _, _, Cnt) ->
+		  Cnt+1
+	  end, 0, V, We),
+    case Mode of
+        more -> Cnt >= N;
+        less -> Cnt =< N;
+        exactly -> Cnt =:= N
+    end.
+
+faces_with({faces_with,specify}, _) ->
+    Qs = [{vframe,
+           [{hframe,[{label,?__(1,"Number of Edges")},
+             {text,2,[{range,{2,1000}}]}]},
+             {hradio,[{?__(3,"More"),more},
+               {?__(4,"Less"),less},
+               {?__(5,"Exactly"),exactly}],exactly}]}],
+    wings_ask:dialog(true,
+        ?__(2,"Select Faces"), [{hframe,Qs}],
+        fun(Res) ->
+            {select,{by,{faces_with,Res}}}
+        end);
+faces_with(Filter, #st{sel=[]}=St) ->
+    Sel = fun(Face, We) ->
+		  faces_with(Filter, Face, We)
+	    end,
+    wings_sel:make(Sel, face, St);
+faces_with(Filter, #st{selmode=Mode}=St0) ->
+    St = if Mode =:= face -> St0; true -> wings_sel_conv:mode(face, St0) end,
+    Sel = wings_sel:fold(fun(Sel0, #we{id=Id}=We, Acc) ->
+				Sel1 = gb_sets:to_list(Sel0),
+				Faces = [Face || Face <- Sel1, faces_with(Filter, Face, We)],
+				case Faces of
+				  [] -> Acc;
+				  _ -> [{Id,gb_sets:from_list(Faces)}|Acc]
+				end
+			end, [], St),
+	{save_state,wings_sel:set(face,Sel,St0)}.
+
+faces_with(Filter, Face, We) ->
+    Vs = wings_face:vertices(Face, We),
+    case Filter of
+      {non_quad,all} -> Vs =/= 4;
+      {non_quad,odd} -> Vs rem 2 =/= 0;
+      {non_quad,even} -> Vs =/= 4 andalso Vs rem 2 =:= 0;
+      {faces_with,5} -> Vs >= 5;
+      {faces_with,N} when is_integer(N)-> Vs =:= N;
+      {faces_with,[N,more]} -> Vs >= N;
+      {faces_with,[N,less]} -> Vs =< N;
+      {faces_with,[N,exactly]} -> Vs =:= N
+    end.
+
+select_nth_ring(#st{selmode=edge}) ->
+    Qs = [{label,?__(1,"Interval")},
+	  {text,2,[{range,{1,1000}}]}],
+    wings_ask:dialog(true,
+	?__(2,"Select Every Nth Edge Ring"), [{hframe,Qs}],
+	fun([Res]) ->
+	    {select,{edge_loop,{nth_edge_ring,Res}}}
+	end);
+select_nth_ring(St) -> {save_state,St}.
\ No newline at end of file
