@@ -3,7 +3,7 @@
 %%
 %%     This module implements the Smooth command for objects and faces.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -144,7 +144,7 @@ face_centers([Face|Fs], We, Acc) ->
     Attrs = wings_va:face_mixed_attrs(Face, We),
     case wings_face:vertex_positions(Face, We) of
 	[_,_] ->
-	    wings_u:error(?__(1,"Face ") ++ integer_to_list(Face) ++
+	    wings_u:error_msg(?__(1,"Face ") ++ integer_to_list(Face) ++
 			  ?__(2," has only two edges."));
 	Positions ->
 	    Center = wings_util:share(e3d_vec:average(Positions)),
