@@ -3,7 +3,7 @@
 %%
 %%     Various utility functions that not obviously fit somewhere else.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -12,7 +12,7 @@
 %%
 
 -module(wings_u).
--export([error/1,error/2,message/1,get_matrices/2,
+-export([error_msg/1,error_msg/2,message/1,get_matrices/2,
 	 geom_windows/0,menu_restriction/2,
 	 yes_no/2,yes_no/3,yes_no_cancel/3,
 	 export_we/2,win_crash/1,crash_log/2,crash_log/3,
@@ -22,16 +22,16 @@
 -include("wings.hrl").
 -import(lists, [member/2,foreach/2,all/2]).
 
--spec error([any()]) -> no_return().
+-spec error_msg([any()]) -> no_return().
 
-error(Message) when is_list(Message) ->
+error_msg(Message) when is_list(Message) ->
     wings_pb:cancel(),
     throw({command_error,Message}).
 
--spec error([any()], [any()]) -> no_return().
+-spec error_msg([any()], [any()]) -> no_return().
 
-error(Format, Arg) ->
-    error(lists:flatten(io_lib:format(Format, Arg))).
+error_msg(Format, Arg) ->
+    error_msg(lists:flatten(io_lib:format(Format, Arg))).
 
 message(Message) ->
     Qs = {vframe,
@@ -68,14 +68,14 @@ yes_no(Question, Yes) ->
 
 yes_no(Question, Yes, No) ->
     Qs = {vframe,
-	  [{label,Question,[{break,45}]},
+	  [{label,Question,[{break,45*?CHAR_WIDTH}]},
 	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
 		    {button,wings_s:no(),yes_no_fun(No),[cancel]}]}]},
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
 yes_no_cancel(Question, Yes, No) ->
     Qs = {vframe,
-	  [{label,Question,[{break,45}]},
+	  [{label,Question,[{break,45*?CHAR_WIDTH}]},
 	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
 		    {button,wings_s:no(),yes_no_fun(No)},
 		    {button,wings_s:cancel(),
@@ -137,16 +137,16 @@ caption(#st{file=undefined}=St) ->
     wings_io:set_title(Caption),
     St;
 caption(#st{saved=true,file=Name}=St) ->
-    Caption = wings() ++ " - " ++ filename:basename(Name),
+    Caption = wings() ++ " - " ++ Name,
     wings_io:set_title(Caption),
     St;
 caption(#st{saved=auto,file=Name}=St) ->
-    Caption = wings() ++ " - " ++ filename:basename(Name) ++
+    Caption = wings() ++ " - " ++ Name ++
 	"* [" ++ ?__(1,"auto-saved") ++ "]",
     wings_io:set_title(Caption),
     St;
 caption(#st{file=Name}=St) ->
-    Caption = wings() ++ " - " ++ filename:basename(Name) ++ "*",
+    Caption = wings() ++ " - " ++ Name ++ "*",
     wings_io:set_title(Caption),
     St.
 
