@@ -3,7 +3,7 @@
 %%
 %%     Various utility functions that not obviously fit somewhere else.
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -82,6 +82,7 @@ key_format(Key, Msg) ->
 
 %% Like io_lib:format/2, but with very restricted format characters.
 %% BUT allows arguments to ~s to be lists containing Unicode characters.
+%% Now allows ~ts for translation of Asian glyphs.
 %%
 %% Format directives allowed: ~s ~p
 
@@ -152,7 +153,7 @@ array_keys(Array) ->
 array_smallest_key(Array) ->
     try
 	array:sparse_foldl(fun(I, _, _) -> throw(I) end, [], Array),
-	erlang:error(empty_array)
+	error(empty_array)
     catch
 	throw:I when is_integer(I) ->
 	    I
@@ -161,7 +162,7 @@ array_smallest_key(Array) ->
 array_greatest_key(Array) ->
     try
 	array:sparse_foldr(fun(I, _, _) -> throw(I) end, [], Array),
-	erlang:error(empty_array)
+	error(empty_array)
     catch
 	throw:I when is_integer(I) ->
 	    I
@@ -246,7 +247,7 @@ format_1("~s"++F, [S0|Args], Acc) ->
 	    true -> 
 		io:format("Bad string formatter for ~p in ~p~n", 
 			  [S0, lists:flatten(reverse(Acc) ++ F)]),
-		erlang:error(badarg)
+		error(badarg)
 	end,
     format_1(F, Args, [S|Acc]);
 format_1("~p"++F, [S0|Args], Acc) ->
@@ -254,6 +255,8 @@ format_1("~p"++F, [S0|Args], Acc) ->
     format_1(F, Args, [S|Acc]);
 format_1("~~"++F, Args, Acc) ->
     format_1(F, Args, [$~|Acc]);
+format_1("~ts"++F, Args, Acc) ->
+    format_1("~s"++F, Args, Acc);
 format_1([C|F], Args, Acc) when C =/= $~ ->
     format_1(F, Args, [C|Acc]);
 format_1([], [], Acc) -> reverse(Acc).
