@@ -3,7 +3,7 @@
 %%
 %%     This module implements "vectors" and the secondary selection mode.
 %%
-%%  Copyright (c) 2002-2009 Bjorn Gustavsson.
+%%  Copyright (c) 2002-2011 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -62,11 +62,13 @@ do_ask_1(_, [], Res, _, St, Cb) ->
     wings_wm:later(build_result(Res, Cb, St)),
     keep;
 do_ask_1(Modes, Do0, Done, Flags, NewSt, Cb) ->
+    wings_menu:kill_menus(),
     Do = add_help_text(Do0),
     Mag = member(magnet, Flags),
     Ss = #ss{cb=Cb,mag=Mag,selmodes=Modes,new_st=NewSt,
 	     f=fun(_, _) -> keep end},
     wings_wm:later({ask_init,Do,Done}),
+    wings_tweak:toggle_draw(false),
     erase_vector(),
     St = wings_wm:get_current_state(),
     {seq,push,get_event(Ss, St)}.
@@ -199,6 +201,15 @@ handle_event_4({new_state,St}, #ss{f=Check}=Ss, _St0) ->
 	[{Vec,{Msg,SwMsg}},{AltVec,_}] ->
 	    get_event(Ss#ss{info=Msg,vec=Vec,alt_vec=AltVec,sw_msg=SwMsg}, St)
     end;
+handle_event_4({update_state,St}, #ss{f=Check}=Ss, _St0) ->
+    case Check(check, St) of
+	{Vec,Msg} -> 
+	    get_event(Ss#ss{info=Msg,vec=Vec,alt_vec=none,sw_msg=none}, St);
+	[{Vec,Msg}] -> 
+	    get_event(Ss#ss{info=Msg,vec=Vec,alt_vec=none,sw_msg=none}, St);
+	[{Vec,{Msg,SwMsg}},{AltVec,_}] ->
+	    get_event(Ss#ss{info=Msg,vec=Vec,alt_vec=AltVec,sw_msg=SwMsg}, St)
+    end;
 handle_event_4(redraw, Ss, St) ->
     redraw(Ss, St),
     keep;
@@ -215,45 +226,34 @@ handle_event_4({action,{view,auto_rotate}}, _, _) ->
 
 %%%% Vector op Highlight Aim
 handle_event_4({action,{view,highlight_aim}}, Ss, St0) ->
-    case wings_pref:get_value(use_temp_sel) of
-      false ->
-	    handle_event_4({action,{view,aim}}, Ss, St0);
-      true ->
-        {{_,Cmd},St1} = wings:highlight_aim_setup(St0),
-        St2 = wings_view:command(Cmd,St1),
-        get_event(Ss, St2)
-    end;
-
+    {{_,Cmd},St1} = wings:highlight_aim_setup(St0),
+    St2 = wings_view:command(Cmd,St1),
+    get_event(Ss, St2);
 %%%% Temporary Select Aim for View commands with no selection
 handle_event_4({action,{view,Cmd}}, Ss, #st{sel=[]}=St0)
   when Cmd == align_to_selection; Cmd == aim; Cmd == frame ->
-    case wings_pref:get_value(use_temp_sel) of
-    false ->
-      St = wings_view:command(Cmd, St0),
-      get_event(Ss, St);
-    true ->
-      {_,X,Y} = wings_wm:local_mouse_state(),
-      case wings_pick:do_pick(X, Y, St0) of
-        {add,_,St} ->
-          St1 = wings_view:command(Cmd,St),
-		  St2 = wings_sel:clear(St1),
-          get_event(Ss, St2);
-        _Other ->
-          St = wings_view:command(Cmd, St0),
-          get_event(Ss, St)
-      end
+    {_,X,Y} = wings_wm:local_mouse_state(),
+    case wings_pick:do_pick(X, Y, St0) of
+      {add,_,St} ->
+        St1 = wings_view:command(Cmd,St),
+        St2 = wings_sel:clear(St1),
+        get_event(Ss, St2);
+      _Other ->
+        St = wings_view:command(Cmd, St0),
+        get_event(Ss, St)
     end;
-
 handle_event_4({action,{view,Cmd}}, Ss, St0) ->
     St = wings_view:command(Cmd, St0),
     get_event(Ss, St);
 handle_event_4({action,{secondary_selection,abort}}, _, _) ->
     erase_vector(),
+    wings_tweak:toggle_draw(true),
     wings_wm:later(revert_state),
     pick_finish(),
     pop;
 handle_event_4(quit, _Ss, _St) ->
     erase_vector(),
+    wings_tweak:toggle_draw(true),
     wings_io:putback_event(quit),
     pop;
 handle_event_4(init_opengl, _, St) ->
@@ -263,19 +263,9 @@ handle_event_4(_Event, Ss, St) ->
     get_event(Ss, St).
 
 temp_selection(X, Y, St0) ->
-    case wings_pref:get_value(use_temp_sel) of
-	false ->
-	    none;
-	true ->
-	    case wings_pick:do_pick(X, Y, St0) of
-		{add,_,#st{sel=[{_,Sel}]}=St} ->
-		    case gb_sets:size(Sel) < 2 orelse
-			wings_pref:get_value(use_super_temp_sel) of
-			true -> St;
-			false -> none
-		    end;
-		_ -> none
-	    end
+    case wings_pick:do_pick(X, Y, St0) of
+	{add,_,St} -> St;
+	_ -> none
     end.
 
 pick_next(Do, Done, #ss{is_axis=true,vec={{_,_,_},{_,_,_}}=Vec}=Ss, St) ->
@@ -284,6 +274,7 @@ pick_next(Do, Done, #ss{is_axis=true,vec={{_,_,_},{_,_,_}}=Vec}=Ss, St) ->
 pick_next(Do, Done, Ss, St) -> pick_next_1(Do, Done, Ss, St).
 
 pick_next_1([], Res, #ss{cb=Cb,new_st=NewSt}, _) ->
+    wings_tweak:toggle_draw(true),
     pick_finish(),
     wings:clear_mode_restriction(),
     wings_wm:later(build_result(Res, Cb, NewSt)),
@@ -596,15 +587,28 @@ get_vec(face, [Face1, Face2], [We0, We1]) ->
        ?__(20,"Use direction between face centers")}}];
 
 %% Use single edge as axis
-get_vec(edge, [Edge], #we{es=Etab,vp=Vtab}=We) ->
+get_vec(edge, [Edge], #we{es=Etab,vp=Vtab,mirror=Mir}=We) ->
     #edge{vs=Va,ve=Vb,lf=Lf,rf=Rf} = array:get(Edge, Etab),
     VaPos = array:get(Va, Vtab),
     VbPos = array:get(Vb, Vtab),
     Vec = e3d_vec:norm_sub(VbPos, VaPos),
     Center = wings_vertex:center([Va,Vb], We),
-    Ln = wings_face:normal(Lf, We),
-    Rn = wings_face:normal(Rf, We),
-    Normal = e3d_vec:norm(e3d_vec:add(Ln, Rn)),
+    Normal = case Mir of
+      Lf ->
+        Mn = wings_face:normal(Mir, We),
+        Rn = wings_face:normal(Rf, We),
+        Cross = e3d_vec:cross(Mn, Rn),
+        e3d_vec:norm(e3d_vec:cross(Cross, Mn));
+      Rf ->
+        Mn = wings_face:normal(Mir, We),
+        Ln = wings_face:normal(Lf, We),
+        Cross = e3d_vec:cross(Mn, Ln),
+        e3d_vec:norm(e3d_vec:cross(Cross, Mn));
+      _ ->
+        Ln = wings_face:normal(Lf, We),
+        Rn = wings_face:normal(Rf, We),
+        e3d_vec:norm(e3d_vec:add(Ln, Rn))
+    end,
     [{{Center,Vec},
       {?__(1,"Edge saved as axis"),?__(2,"Save edge normal")}},
      {{Center,Normal},
@@ -647,7 +651,7 @@ get_vec(edge, Edges, #we{vp=Vtab}=We) ->
 
 %% Vertex normal
 get_vec(vertex, [V], We) ->
-    Vec = wings_vertex:normal(V, We),
+    Vec = vertex_no_mirror_norm(V,We),
     Center = wings_vertex:center([V], We),
     [{{Center,Vec},?__(8,"Vertex normal saved.")}];
 %% Direction between 2 vertices as axis
@@ -656,8 +660,8 @@ get_vec(vertex, [Va,Vb]=Vs, We) ->
     VbPos = wings_vertex:pos(Vb, We),
     Vec = e3d_vec:norm_sub(VaPos, VbPos),
     Center = wings_vertex:center(Vs, We),
-    Normal = e3d_vec:norm(e3d_vec:add(wings_vertex:normal(Va, We),
-				      wings_vertex:normal(Vb, We))),
+    Normal = e3d_vec:norm(e3d_vec:add(vertex_no_mirror_norm(Va,We),
+        vertex_no_mirror_norm(Vb,We))),
     [{{Center,Vec},
       {?__(9,"Direction between vertices saved as axis."),
        ?__(10,"Use average of vertex normals as axis")}},
@@ -722,28 +726,18 @@ get_vec(_, _, _) -> {none,?__(23,"Select vertices, edges, or faces.")}.
 check_point(#st{sel=[]}) -> {none,""};
 check_point(St) ->
     case kill_mirror(St) of
-	St ->
-	    Center = e3d_vec:average(wings_sel:bounding_box(St)),
-	    [{Center,?__(1,"Midpoint of selection saved.")}];
-	NoMirror ->
-	    Center = e3d_vec:average(wings_sel:bounding_box(St)),
-	    NoMirrorCenter = e3d_vec:average(wings_sel:bounding_box(NoMirror)),
-            UseMirrorPref = wings_pref:get_value(use_mirror_for_sels),
-            if UseMirrorPref ->
-              [{Center,
-                {?__(1,"Midpoint of selection saved."),
-                 ?__(2,"Disregard virtual mirror in reference point calculation")}},
-               {NoMirrorCenter,
-                {?__(1,"Midpoint of selection saved."),
-                 ?__(3,"Include virtual mirror in reference point calculation")}}];
-            true ->
-              [{NoMirrorCenter,
-                {?__(1,"Midpoint of selection saved."),
-                 ?__(3,"Include virtual mirror in reference point calculation")}},
-               {Center,
-                {?__(1,"Midpoint of selection saved."),
-                 ?__(2,"Disregard virtual mirror in reference point calculation")}}]
-            end
+        St ->
+            Center = e3d_vec:average(wings_sel:bounding_box(St)),
+            [{Center,?__(1,"Midpoint of selection saved.")}];
+        NoMirror ->
+            Center = e3d_vec:average(wings_sel:bounding_box(St)),
+            NoMirrorCenter = e3d_vec:average(wings_sel:bounding_box(NoMirror)),
+            [{NoMirrorCenter,
+              {?__(1,"Midpoint of selection saved."),
+               ?__(3,"Include virtual mirror in reference point calculation")}},
+             {Center,
+              {?__(1,"Midpoint of selection saved."),
+               ?__(2,"Disregard virtual mirror in reference point calculation")}}]
     end.
 
 check_magnet_point(#st{sel=[]}) -> {none,""};
@@ -872,3 +866,18 @@ make_vec_dlist_1(Center) ->
     gl:'begin'(?GL_POINTS),
     gl:vertex3fv(Center),
     gl:'end'().
+
+vertex_no_mirror_norm(V, #we{mirror=Mir}=We) ->
+    {Mn,Ns} = wings_vertex:fold(fun(_, Face, _, {M,A}) when Face =/= Mir ->
+          {M,[wings_face:normal(Face, We)|A]};
+          (_,_,_,{_,A}) ->
+            {wings_face:normal(Mir, We),A}
+      end, {none,[]}, V, We),
+    N = e3d_vec:norm(e3d_vec:add(Ns)),
+    case Mn of
+      none ->
+        N;
+      Mn ->
+        Cross = e3d_vec:cross(Mn,N),
+        e3d_vec:cross(Cross,Mn)
+    end.
