@@ -5,7 +5,7 @@
 %%     we records (winged-edged records, the central data structure
 %%     in Wings 3D).
 %%
-%%  Copyright (c) 2001-2009 Bjorn Gustavsson
+%%  Copyright (c) 2001-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -14,7 +14,8 @@
 %%
 
 -module(wings_we).
--export([build/2,rebuild/1,fast_rebuild/1,
+-export([map/2, 
+	 build/2,rebuild/1,fast_rebuild/1,
 	 new_wrap_range/3,id/2,bump_id/1,
 	 new_id/1,new_ids/2,
 	 invert_normals/1,
@@ -38,12 +39,19 @@
 -include("wings.hrl").
 -include("e3d.hrl").
 -import(lists, [foreach/2,foldl/3,sort/1,keysort/2,reverse/1,zip/2,partition/2]).
--import(erlang, [max/2]).
 
 %%%
 %%% API.
 %%%
 
+%% Apply fun on all we's.
+map(Fun, St = #st{shapes=Shs0}) ->
+    Objs0 = lists:map(Fun, gb_trees:values(Shs0)),
+    Shs = gb_trees:from_orddict([{We#we.id, We} || We <- Objs0]),
+    St#st{shapes=Shs}.
+
+%% build() -> We'
+%% Create a we from faces and vertices or a mesh.
 build(Mode, #e3d_mesh{fs=Fs0,vs=Vs,tx=Tx,he=He}) when is_atom(Mode) ->
     Fs = translate_faces(Fs0, list_to_tuple(Tx), []),
     wings_we_build:we(Fs, Vs, He);
@@ -591,7 +599,7 @@ merge_plugins(Wes) ->
 		    catch _:_ -> Acc
 		    end
 	    end,
-    Merged = lists:foldl(Merge, [], PMods),
+    Merged = lists:reverse(lists:foldl(Merge, [], PMods)),
     gb_trees:from_orddict(Merged).
 		    
 merge_renumber(Wes0) ->
@@ -1165,7 +1173,7 @@ validate_face(Face, Edge, Etab) ->
     validate_face_vertices(Vs, V).
 
 validate_face_vertices([V|_], V) ->
-    erlang:error(repeated_vertex);
+    error(repeated_vertex);
 validate_face_vertices([_], _) ->
     true;
 validate_face_vertices([V|Vs], _) ->
