@@ -3,7 +3,7 @@
 %%
 %%     Window manager for Wings.
 %%
-%%  Copyright (c) 2002-2009 Bjorn Gustavsson
+%%  Copyright (c) 2002-2011 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -15,22 +15,22 @@
 -export([toplevel/6,toplevel_title/1,toplevel_title/2,set_knob/3]).
 -export([init/0,enter_event_loop/0,dirty/0,dirty_mode/1,pdirty/0,
 	 reinit_opengl/0,
-	 new/4,delete/1,raise/1,
+	 new/4,delete/1,raise/1,rollup/2,
 	 link/2,hide/1,show/1,is_hidden/1,
 	 later/1,send/2,psend/2,send_after_redraw/2,
 	 set_timer/2,cancel_timer/1,
 	 this/0,offset/3,move/2,move/3,resize/2,pos/1,windows/0,is_window/1,
-	 window_below/2,
+	 window_below/2,resize_windows/2,
 	 update_window/2,clear_background/0,
 	 callback/1,current_state/1,get_current_state/0,notify/1,
 	 local2global/1,local2global/2,global2local/2,local_mouse_state/0,
-	 translation_change/0]).
+	 translation_change/0,is_geom/0]).
 
 %% Window information.
 -export([top_size/0,viewport/0,viewport/1,
 	 win_size/0,win_ul/0,win_rect/0,
 	 win_size/1,win_ul/1,win_ur/1,win_ll/1,win_lr/1,win_z/1,
-	 win_rect/1]).
+	 win_rect/1,win_rollup/1]).
 
 %% Focus management.
 -export([grab_focus/0,grab_focus/1,release_focus/0,
@@ -67,7 +67,8 @@
 	 name,					%Name of window.
 	 stk,					%Event handler stack.
 	 props,					%Window properties.
-	 links=[]			  %Windows linked to this one.
+	 links=[],			  %Windows linked to this one.
+	 rollup=false			%Rollup window into Title Bar {controller,Name}
 	}).
 
 -record(se,					%Stack entry record.
@@ -239,7 +240,6 @@ delete_windows(Name, W0) ->
 		     (_, A) -> A
 		  end, W, Links)
     end.
-
 raise(Name) ->
     case get_window_data(Name) of
 	#win{z=Z} when Z < ?Z_LOWEST_DYNAMIC -> ok;
@@ -251,6 +251,73 @@ raise(Name) ->
 	    end
     end.
 
+rollup(Action, Name) ->
+    case (Action =:= rolldown) =:= win_rollup(Name) of
+      true ->
+        WinData = gb_trees:values(get(wm_windows)),
+        rollup(Action, WinData, Name);
+      false ->
+        ok
+    end.
+
+rollup(Action, [#win{name={closer,Name}}|T], Name) ->
+    rollup(Action,T, Name);
+rollup(Action, [#win{name={controller,Name}}|T], Name) when Name /= geom ->
+    rollup(Action, T, Name);
+rollup(rollup, [#win{name=geom}=W|T], geom) ->
+    put_window_data(geom, W#win{z=?Z_LOWEST_DYNAMIC, rollup=true}),
+    rollup(rollup, T, geom);
+rollup(rolldown, [#win{name=geom}=W|T], geom) ->
+    put_window_data(geom, W#win{rollup=false}),
+    rollup(rolldown, T, geom);
+rollup(rollup, [#win{name={object,geom}}|T], geom) ->
+    rollup(rollup, T, geom);
+
+rollup(rollup, [#win{z=Z,name={toolbar,Name}=Win}=W|T], Name) when Name /= geom ->
+    case Z < -1 of
+      true -> ok;
+      false -> 
+        put_window_data(Win, W#win{z=-1})
+    end,
+    rollup(rollup, T, Name);
+rollup(rolldown, [#win{z=Z,name={toolbar,Name}=Win}=W|T], Name) when Name /= geom ->
+    case Z < -1 of
+      true -> ok;
+      false -> 
+        put_window_data(Win, W#win{z=win_z({controller,Name})})
+    end,
+    rollup(rolldown, T, Name);
+
+rollup(Action, [#win{z=Z,name={toolbar,geom}=Win}=W|T], geom) ->
+    case Z < 0 of
+     true -> ok;
+     false ->
+       put_window_data(Win, W#win{z=?Z_LOWEST_DYNAMIC})
+    end,
+    rollup(Action, T, geom);
+
+rollup(rollup, [#win{name={_,geom}=Win}=W|T], geom) ->
+    put_window_data(Win, W#win{z=?Z_LOWEST_DYNAMIC, rollup=true}),
+    rollup(rollup, T, geom);
+rollup(rolldown, [#win{name={_,geom}}|T], geom) ->
+    rollup(rolldown, T, geom);
+rollup(rollup, [#win{z=Z,name=Name}=W|T], Name) ->
+    put_window_data(Name, W#win{z=-Z, rollup=true}),
+    rollup(rollup, T, Name);
+rollup(rolldown, [#win{z=Z,name=Name}=W|T], Name) ->
+    put_window_data(Name, W#win{z=-Z, rollup=false}),
+    rollup(rolldown, T, Name);
+rollup(rollup, [#win{name={_,Name}=Win}|T], Name) ->
+    hide(Win),
+    rollup(rollup, T, Name);
+rollup(rolldown, [#win{name={_,Name}=Win}|T], Name) ->
+    show(Win),
+    rollup(rolldown, T, Name);
+rollup(Action, [_|T], Name) ->
+    rollup(Action, T, Name);
+rollup(_, [], _) ->
+    dirty().
+
 highest_z() ->
     highest_z_1(gb_trees:values(get(wm_windows)), 0).
 
@@ -293,6 +360,13 @@ this() ->
 	Active -> Active
     end.
 
+is_geom() ->
+    case this() of
+      geom -> true;
+      {geom,_} -> true;
+      _ -> false
+    end.
+
 windows() ->	    
     gb_trees:keys(get(wm_windows)).
 
@@ -360,6 +434,8 @@ update_window_1([{w,W}|T], Win) ->
     update_window_1(T, Win#win{w=W});
 update_window_1([{h,H}|T], Win) ->
     update_window_1(T, Win#win{h=H});
+update_window_1([{rollup,Rollup}|T], Win) ->
+    update_window_1(T, Win#win{rollup=Rollup});
 update_window_1([], Win) ->
     range_check(Win).
 
@@ -448,6 +524,11 @@ win_rect(Name) ->
     #win{x=X,y=Y,w=W,h=H} = get_window_data(Name),
     {{X,Y},{W,H}}.
 
+win_rollup({dialog,_}) -> no;
+win_rollup(Name) ->
+    #win{rollup=Rollup} = get_window_data(Name),
+    Rollup.
+
 local2global(#mousebutton{x=X0,y=Y0}=Ev) ->
     {X,Y} = local2global(X0, Y0),
     Ev#mousebutton{x=X,y=Y};
@@ -586,11 +667,11 @@ update_focus(Active) ->
 	    do_dispatch(Active, got_focus)
     end.
 
-resize_windows(W, H) ->    
+resize_windows(W, H) ->
     Event = #resize{w=W,h=H},
 
     MsgData0 = get_window_data(message),
-    MsgH = ?CHAR_HEIGHT+7,
+    MsgH = ?CHAR_HEIGHT+9,
     MsgY = H-MsgH,
     MsgData1 = MsgData0#win{x=0,y=MsgY,w=W,h=MsgH},
     put_window_data(message, MsgData1),
@@ -1087,8 +1168,8 @@ message_event(_) -> keep.
 message_redraw(Msg, Right) ->
     {W,_} = message_setup(),
     if
-	Msg == [] -> ok;
-	true -> wings_io:text_at(0, Msg)
+	Msg =:= [] -> ok;
+	true -> wings_io:text_at(0, -1, Msg)
     end,
     OsType = get(wings_os_type),
     RMarg0 = case OsType of
@@ -1098,18 +1179,19 @@ message_redraw(Msg, Right) ->
     case Right of
 	[] -> ok;
 	_ ->
-	    Cw = wings_text:width(),
-	    MsgW = wings_text:width(Msg),
+	    Col = e3d_vec:mul(wings_pref:get_value(info_line_bg),0.6),
+	    Cw = ?CHAR_WIDTH,
+	    CH = ?CHAR_HEIGHT,
+	    Lh = ?LINE_HEIGHT,
+	    MsgW = wings_text:width(lists:flatten(Msg)),
 	    RightW = wings_text:width(Right),
 	    RMarg = RMarg0 + 2*Cw,
-	    if 
+	    if
 		MsgW+RightW < W - RMarg ->
 		    Pos = W - RightW - RMarg,
-		    wings_io:set_color(e3d_vec:mul(wings_pref:get_value(info_line_bg),0.9)),
-		    gl:recti(Pos-Cw, 1-wings_text:height(),
-			     Pos+RightW+Cw, 3),
+		    wings_io:sunken_rect(Pos-12, -CH, RightW+12, Lh, Col, Col),
 		    wings_io:set_color(wings_pref:get_value(info_line_text)),
-		    wings_io:text_at(Pos, Right);
+		    wings_io:text_at(Pos-6, -1, Right);
 		true -> ok
 	    end
     end,
@@ -1125,7 +1207,7 @@ message_redraw(Msg, Right) ->
 message_setup() ->
     wings_io:ortho_setup(none),
     {W,H} = win_size(),
-    wings_io:gradient_rect(0, 0, W, H, wings_pref:get_value(info_line_bg)),
+    wings_io:gradient_rect_burst(0, 0, W, H, wings_pref:get_value(info_line_bg)),
     wings_io:set_color(wings_pref:get_value(info_line_text)),
     gl:translatef(10, H-5.375, 0),
     {W,H}.
